@@ -2,6 +2,7 @@
 '''
 Module to manage Linux kernel modules
 '''
+from __future__ import absolute_import
 
 # Import python libs
 import os
@@ -15,7 +16,7 @@ def __virtual__():
     '''
     Only runs on Linux systems
     '''
-    return 'kmod' if __grains__['kernel'] == 'Linux' else False
+    return __grains__['kernel'] == 'Linux'
 
 
 def _new_mods(pre_mods, post_mods):
@@ -46,19 +47,12 @@ def _rm_mods(pre_mods, post_mods):
     return pre - post
 
 
-def _union_module(a, b):
-    '''
-    Return union of two list where duplicated items are only once
-    '''
-    return list(set(a) | set(b))
-
-
 def _get_modules_conf():
     '''
     Return location of modules config file.
     Default: /etc/modules
     '''
-    if __grains__['os'] == 'Arch':
+    if 'systemd' in __grains__:
         return '/etc/modules-load.d/salt_managed.conf'
     return '/etc/modules'
 
@@ -84,10 +78,11 @@ def _set_persistent_module(mod):
     if not os.path.exists(conf):
         __salt__['file.touch'](conf)
     mod_name = _strip_module_name(mod)
-    if not mod_name or mod_name in mod_list(True) or mod_name not in available():
+    if not mod_name or mod_name in mod_list(True) or mod_name \
+            not in available():
         return set()
     escape_mod = re.escape(mod)
-    ## If module is commented only uncomment it
+    # If module is commented only uncomment it
     if __salt__['file.contains_regex_multiline'](conf,
                                                  '^#[\t ]*{0}[\t ]*$'.format(
                                                      escape_mod)):
@@ -219,7 +214,8 @@ def load(mod, persist=False):
         salt '*' kmod.load kvm
     '''
     pre_mods = lsmod()
-    response = __salt__['cmd.run_all']('modprobe {0}'.format(mod))
+    response = __salt__['cmd.run_all']('modprobe {0}'.format(mod),
+                                       python_shell=False)
     if response['retcode'] == 0:
         post_mods = lsmod()
         mods = _new_mods(pre_mods, post_mods)
@@ -265,7 +261,7 @@ def remove(mod, persist=False, comment=True):
         salt '*' kmod.remove kvm
     '''
     pre_mods = lsmod()
-    __salt__['cmd.run_all']('modprobe -r {0}'.format(mod))
+    __salt__['cmd.run_all']('rmmod {0}'.format(mod), python_shell=False)
     post_mods = lsmod()
     mods = _rm_mods(pre_mods, post_mods)
     persist_mods = set()
