@@ -2,6 +2,7 @@
 '''
 Module for viewing and modifying sysctl parameters
 '''
+from __future__ import absolute_import
 
 # Import python libs
 import logging
@@ -10,9 +11,10 @@ import re
 
 # Import salt libs
 import salt.utils
-from salt._compat import string_types
+from salt.ext.six import string_types
 from salt.exceptions import CommandExecutionError
 from salt.modules.systemd import _sd_booted
+import string
 
 log = logging.getLogger(__name__)
 
@@ -34,6 +36,16 @@ def __virtual__():
     return __virtualname__
 
 
+def _check_systemd_salt_config():
+    conf = '/etc/sysctl.d/99-salt.conf'
+    if not os.path.exists(conf):
+        sysctl_dir = os.path.split(conf)[0]
+        if not os.path.exists(sysctl_dir):
+            os.makedirs(sysctl_dir)
+        salt.utils.fopen(conf, 'w').close()
+    return conf
+
+
 def default_config():
     '''
     Linux hosts using systemd 207 or later ignore ``/etc/sysctl.conf`` and only
@@ -47,7 +59,7 @@ def default_config():
 
         salt -G 'kernel:Linux' sysctl.default_config
     '''
-    if _sd_booted():
+    if _sd_booted(__context__):
         for line in __salt__['cmd.run_stdout'](
             'systemctl --version'
         ).splitlines():
@@ -55,7 +67,7 @@ def default_config():
                 version = line.split()[-1]
                 try:
                     if int(version) >= 207:
-                        return '/etc/sysctl.d/99-salt.conf'
+                        return _check_systemd_salt_config()
                 except ValueError:
                     log.error(
                         'Unexpected non-numeric systemd version {0!r} '
@@ -66,24 +78,43 @@ def default_config():
     return '/etc/sysctl.conf'
 
 
-def show():
+def show(config_file=False):
     '''
     Return a list of sysctl parameters for this minion
 
+    config: Pull the data from the system configuration file
+        instead of the live data.
+
     CLI Example:
 
     .. code-block:: bash
 
         salt '*' sysctl.show
     '''
-    cmd = 'sysctl -a'
     ret = {}
-    out = __salt__['cmd.run_stdout'](cmd, output_loglevel='trace')
-    for line in out.splitlines():
-        if not line or ' = ' not in line:
-            continue
-        comps = line.split(' = ', 1)
-        ret[comps[0]] = comps[1]
+    if config_file:
+        try:
+            for line in salt.utils.fopen(config_file):
+                if not line.startswith('#') and '=' in line:
+                    # search if we have some '=' instead of ' = ' separators
+                    SPLIT = ' = '
+                    if SPLIT not in line:
+                        SPLIT = SPLIT.strip()
+                    key, value = line.split(SPLIT, 1)
+                    key = key.strip()
+                    value = value.lstrip()
+                    ret[key] = value
+        except (OSError, IOError):
+            log.error('Could not open sysctl file')
+            return None
+    else:
+        cmd = 'sysctl -a'
+        out = __salt__['cmd.run_stdout'](cmd, output_loglevel='trace')
+        for line in out.splitlines():
+            if not line or ' = ' not in line:
+                continue
+            comps = line.split(' = ', 1)
+            ret[comps[0]] = comps[1]
     return ret
 
 
@@ -98,7 +129,7 @@ def get(name):
         salt '*' sysctl.get net.ipv4.ip_forward
     '''
     cmd = 'sysctl -n {0}'.format(name)
-    out = __salt__['cmd.run'](cmd)
+    out = __salt__['cmd.run'](cmd, python_shell=False)
     return out
 
 
@@ -113,23 +144,24 @@ def assign(name, value):
         salt '*' sysctl.assign net.ipv4.ip_forward 1
     '''
     value = str(value)
-    sysctl_file = '/proc/sys/{0}'.format(name.replace('.', '/'))
+    sysctl_file = '/proc/sys/{0}'.format(name.translate(string.maketrans('./', '/.')))
     if not os.path.exists(sysctl_file):
         raise CommandExecutionError('sysctl {0} does not exist'.format(name))
 
     ret = {}
     cmd = 'sysctl -w {0}="{1}"'.format(name, value)
-    data = __salt__['cmd.run_all'](cmd)
+    data = __salt__['cmd.run_all'](cmd, python_shell=False)
     out = data['stdout']
+    err = data['stderr']
 
     # Example:
     #    # sysctl -w net.ipv4.tcp_rmem="4096 87380 16777216"
     #    net.ipv4.tcp_rmem = 4096 87380 16777216
     regex = re.compile(r'^{0}\s+=\s+{1}$'.format(re.escape(name), re.escape(value)))
 
-    if not regex.match(out):
-        if data['retcode'] != 0 and data['stderr']:
-            error = data['stderr']
+    if not regex.match(out) or 'Invalid argument' in str(err):
+        if data['retcode'] != 0 and err:
+            error = err
         else:
             error = out
         raise CommandExecutionError('sysctl -w failed: {0}'.format(error))
