@@ -2,11 +2,17 @@
 '''
 Manage PHP pecl extensions.
 '''
+from __future__ import absolute_import
 
 # Import python libs
 import re
 import logging
 
+try:
+    from shlex import quote as _cmd_quote  # pylint: disable=E0611
+except ImportError:
+    from pipes import quote as _cmd_quote
+
 # Import salt libs
 import salt.utils
 
@@ -27,9 +33,9 @@ def _pecl(command, defaults=False):
     '''
     cmdline = 'pecl {0}'.format(command)
     if salt.utils.is_true(defaults):
-        cmdline = "printf '\n' | " + cmdline
+        cmdline = 'yes ' "''" + ' | ' + cmdline
 
-    ret = __salt__['cmd.run_all'](cmdline)
+    ret = __salt__['cmd.run_all'](cmdline, python_shell=True)
 
     if ret['retcode'] == 0:
         return ret['stdout']
@@ -61,12 +67,12 @@ def install(pecls, defaults=False, force=False, preferred_state='stable'):
 
         salt '*' pecl.install fuse
     '''
-    preferred_state = '-d preferred_state={0}'.format(preferred_state)
+    preferred_state = '-d preferred_state={0}'.format(_cmd_quote(preferred_state))
     if force:
-        return _pecl('{0} install -f {1}'.format(preferred_state, pecls),
+        return _pecl('{0} install -f {1}'.format(preferred_state, _cmd_quote(pecls)),
                      defaults=defaults)
     else:
-        _pecl('{0} install {1}'.format(preferred_state, pecls),
+        _pecl('{0} install {1}'.format(preferred_state, _cmd_quote(pecls)),
               defaults=defaults)
         if not isinstance(pecls, list):
             pecls = [pecls]
@@ -102,7 +108,7 @@ def uninstall(pecls):
 
         salt '*' pecl.uninstall fuse
     '''
-    return _pecl('uninstall {0}'.format(pecls))
+    return _pecl('uninstall {0}'.format(_cmd_quote(pecls)))
 
 
 def update(pecls):
@@ -118,7 +124,7 @@ def update(pecls):
 
         salt '*' pecl.update fuse
     '''
-    return _pecl('install -U {0}'.format(pecls))
+    return _pecl('install -U {0}'.format(_cmd_quote(pecls)))
 
 
 def list_(channel=None):
@@ -131,22 +137,16 @@ def list_(channel=None):
 
         salt '*' pecl.list
     '''
+    pecl_channel_pat = re.compile('^([^ ]+)[ ]+([^ ]+)[ ]+([^ ]+)')
     pecls = {}
     command = 'list'
     if channel:
-        command = '{0} -c {1}'.format(command, channel)
+        command = '{0} -c {1}'.format(command, _cmd_quote(channel))
     lines = _pecl(command).splitlines()
-    lines.pop(0)
-    # Only one line if no package installed:
-    # (no packages installed from channel pecl.php.net)
-    if not lines:
-        return pecls
-
-    lines.pop(0)
-    lines.pop(0)
+    lines = (l for l in lines if pecl_channel_pat.match(l))
 
     for line in lines:
-        match = re.match('^([^ ]+)[ ]+([^ ]+)[ ]+([^ ]+)', line)
+        match = pecl_channel_pat.match(line)
         if match:
             pecls[match.group(1)] = [match.group(2), match.group(3)]
 
