@@ -4,15 +4,16 @@ Module to provide RabbitMQ compatibility to Salt.
 Todo: A lot, need to add cluster support, logging, and minion configuration
 data.
 '''
+from __future__ import absolute_import
 
 # Import salt libs
 import salt.utils
-from salt.utils import decorators
 
 # Import python libs
 import logging
 import random
 import string
+from salt.ext.six.moves import range
 
 log = logging.getLogger(__name__)
 
@@ -38,6 +39,42 @@ def _format_response(response, msg):
     }
 
 
+def _get_rabbitmq_plugin():
+    '''
+    Returns the rabbitmq-plugin command path if we're running an OS that
+    doesn't put it in the standard /usr/bin or /usr/local/bin
+    This works by taking the rabbitmq-server version and looking for where it
+    seems to be hidden in /usr/lib.
+    '''
+    rabbitmq = salt.utils.which('rabbitmq-plugins')
+
+    if rabbitmq is None:
+        version = __salt__['pkg.version']('rabbitmq-server').split('-')[0]
+
+        rabbitmq = ('/usr/lib/rabbitmq/lib/rabbitmq_server-{0}'
+                    '/sbin/rabbitmq-plugins').format(version)
+
+    return rabbitmq
+
+
+def _output_to_dict(cmdoutput, values_mapper=None):
+    '''Convert rabbitmqctl output to a dict of data
+    cmdoutput: string output of rabbitmqctl commands
+    values_mapper: function object to process the values part of each line
+    '''
+    ret = {}
+    if values_mapper is None:
+        values_mapper = lambda string: string.split('\t')
+
+    # remove first and last line: Listing ... - ...done
+    data_rows = cmdoutput.splitlines()[1:-1]
+    for row in data_rows:
+        key, values = row.split('\t', 1)
+        ret[key] = values_mapper(values)
+
+    return ret
+
+
 def list_users(runas=None):
     '''
     Return a list of users based off of rabbitmqctl user_list.
@@ -48,17 +85,14 @@ def list_users(runas=None):
 
         salt '*' rabbitmq.list_users
     '''
-    ret = {}
+    if runas is None:
+        runas = salt.utils.get_user()
     res = __salt__['cmd.run']('rabbitmqctl list_users',
                               runas=runas)
-    for line in res.splitlines():
-        if '...' not in line or line == '\n':
-            parts = line.split('\t')
-            if len(parts) < 2:
-                continue
-            user, properties = parts[0], parts[1]
-            ret[user] = properties
-    return ret
+
+    # func to get tags from string such as "[admin, monitoring]"
+    func = lambda string: set(string[1:-1].split(','))
+    return _output_to_dict(res, func)
 
 
 def list_vhosts(runas=None):
@@ -71,11 +105,13 @@ def list_vhosts(runas=None):
 
         salt '*' rabbitmq.list_vhosts
     '''
+    if runas is None:
+        runas = salt.utils.get_user()
     res = __salt__['cmd.run']('rabbitmqctl list_vhosts',
                               runas=runas)
-    lines = res.splitlines()
-    vhost_list = [line for line in lines if '...' not in line]
-    return vhost_list
+
+    # remove first and last line: Listing ... - ...done
+    return res.splitlines()[1:-1]
 
 
 def user_exists(name, runas=None):
@@ -88,6 +124,8 @@ def user_exists(name, runas=None):
 
         salt '*' rabbitmq.user_exists rabbit_user
     '''
+    if runas is None:
+        runas = salt.utils.get_user()
     user_list = list_users(runas=runas)
     log.debug(user_list)
 
@@ -104,6 +142,8 @@ def vhost_exists(name, runas=None):
 
         salt '*' rabbitmq.vhost_exists rabbit_host
     '''
+    if runas is None:
+        runas = salt.utils.get_user()
     return name in list_vhosts(runas=runas)
 
 
@@ -122,18 +162,22 @@ def add_user(name, password=None, runas=None):
     if password is None:
         # Generate a random, temporary password. RabbitMQ requires one.
         clear_pw = True
-        password = ''.join(random.choice(
+        password = ''.join(random.SystemRandom().choice(
             string.ascii_uppercase + string.digits) for x in range(15))
 
+    if runas is None:
+        runas = salt.utils.get_user()
     res = __salt__['cmd.run'](
         'rabbitmqctl add_user {0} {1!r}'.format(name, password),
+        output_loglevel='quiet',
+        python_shell=False,
         runas=runas)
 
     if clear_pw:
         # Now, Clear the random password from the account, if necessary
         res2 = clear_password(name, runas)
 
-        if 'Error' in res2.keys():
+        if 'Error' in res2:
             # Clearing the password failed. We should try to cleanup
             # and rerun and error.
             delete_user(name, runas)
@@ -154,7 +198,10 @@ def delete_user(name, runas=None):
 
         salt '*' rabbitmq.delete_user rabbit_user
     '''
+    if runas is None:
+        runas = salt.utils.get_user()
     res = __salt__['cmd.run']('rabbitmqctl delete_user {0}'.format(name),
+                              python_shell=False,
                               runas=runas)
     msg = 'Deleted'
 
@@ -171,8 +218,12 @@ def change_password(name, password, runas=None):
 
         salt '*' rabbitmq.change_password rabbit_user password
     '''
+    if runas is None:
+        runas = salt.utils.get_user()
     res = __salt__['cmd.run'](
         'rabbitmqctl change_password {0} {1!r}'.format(name, password),
+        output_loglevel='quiet',
+        python_shell=False,
         runas=runas)
     msg = 'Password Changed'
 
@@ -189,7 +240,10 @@ def clear_password(name, runas=None):
 
         salt '*' rabbitmq.clear_password rabbit_user
     '''
+    if runas is None:
+        runas = salt.utils.get_user()
     res = __salt__['cmd.run']('rabbitmqctl clear_password {0}'.format(name),
+                              python_shell=False,
                               runas=runas)
     msg = 'Password Cleared'
 
@@ -206,7 +260,10 @@ def add_vhost(vhost, runas=None):
 
         salt '*' rabbitmq add_vhost '<vhost_name>'
     '''
+    if runas is None:
+        runas = salt.utils.get_user()
     res = __salt__['cmd.run']('rabbitmqctl add_vhost {0}'.format(vhost),
+                              python_shell=False,
                               runas=runas)
 
     msg = 'Added'
@@ -223,14 +280,16 @@ def delete_vhost(vhost, runas=None):
 
         salt '*' rabbitmq.delete_vhost '<vhost_name>'
     '''
+    if runas is None:
+        runas = salt.utils.get_user()
     res = __salt__['cmd.run']('rabbitmqctl delete_vhost {0}'.format(vhost),
+                              python_shell=False,
                               runas=runas)
     msg = 'Deleted'
     return _format_response(res, msg)
 
 
-def set_permissions(vhost, user, conf='.*', write='.*', read='.*',
-        runas=None):
+def set_permissions(vhost, user, conf='.*', write='.*', read='.*', runas=None):
     '''
     Sets permissions for vhost via rabbitmqctl set_permissions
 
@@ -240,9 +299,12 @@ def set_permissions(vhost, user, conf='.*', write='.*', read='.*',
 
         salt '*' rabbitmq.set_permissions 'myvhost' 'myuser'
     '''
+    if runas is None:
+        runas = salt.utils.get_user()
     res = __salt__['cmd.run'](
         'rabbitmqctl set_permissions -p {0} {1} "{2}" "{3}" "{4}"'.format(
             vhost, user, conf, write, read),
+        python_shell=False,
         runas=runas)
     msg = 'Permissions Set'
     return _format_response(res, msg)
@@ -258,13 +320,17 @@ def list_permissions(vhost, runas=None):
 
         salt '*' rabbitmq.list_permissions '/myvhost'
     '''
+    if runas is None:
+        runas = salt.utils.get_user()
     res = __salt__['cmd.run'](
         'rabbitmqctl list_permissions -p {0}'.format(vhost),
+        python_shell=False,
         runas=runas)
-    return [r.split('\t') for r in res.splitlines()]
+
+    return _output_to_dict(res)
 
 
-def list_user_permissions(name, user=None):
+def list_user_permissions(name, runas=None):
     '''
     List permissions for a user via rabbitmqctl list_user_permissions
 
@@ -274,14 +340,18 @@ def list_user_permissions(name, user=None):
 
         salt '*' rabbitmq.list_user_permissions 'user'.
     '''
+    if runas is None:
+        runas = salt.utils.get_user()
     res = __salt__['cmd.run'](
         'rabbitmqctl list_user_permissions {0}'.format(name),
-        runas=user)
-    return [r.split('\t') for r in res.splitlines()]
+        python_shell=False,
+        runas=runas)
+
+    return _output_to_dict(res)
 
 
 def set_user_tags(name, tags, runas=None):
-    '''Add user tags via rabbitctl set_user_tags
+    '''Add user tags via rabbitmqctl set_user_tags
 
     CLI Example:
 
@@ -289,14 +359,21 @@ def set_user_tags(name, tags, runas=None):
 
         salt '*' rabbitmq.set_user_tags 'myadmin' 'administrator'
     '''
+    if runas is None:
+        runas = salt.utils.get_user()
+
+    if tags and isinstance(tags, (list, tuple)):
+        tags = ' '.join(tags)
+
     res = __salt__['cmd.run'](
         'rabbitmqctl set_user_tags {0} {1}'.format(name, tags),
+        python_shell=False,
         runas=runas)
     msg = "Tag(s) set"
     return _format_response(res, msg)
 
 
-def status(user=None):
+def status(runas=None):
     '''
     return rabbitmq status
 
@@ -306,14 +383,16 @@ def status(user=None):
 
         salt '*' rabbitmq.status
     '''
+    if runas is None:
+        runas = salt.utils.get_user()
     res = __salt__['cmd.run'](
         'rabbitmqctl status',
-        runas=user
+        runas=runas
     )
     return res
 
 
-def cluster_status(user=None):
+def cluster_status(runas=None):
     '''
     return rabbitmq cluster_status
 
@@ -323,14 +402,16 @@ def cluster_status(user=None):
 
         salt '*' rabbitmq.cluster_status
     '''
+    if runas is None:
+        runas = salt.utils.get_user()
     res = __salt__['cmd.run'](
         'rabbitmqctl cluster_status',
-        runas=user)
+        runas=runas)
 
     return res
 
 
-def join_cluster(host, user='rabbit', runas=None):
+def join_cluster(host, user='rabbit', ram_node=None, runas=None):
     '''
     Join a rabbit cluster
 
@@ -338,13 +419,17 @@ def join_cluster(host, user='rabbit', runas=None):
 
     .. code-block:: bash
 
-        salt '*' rabbitmq.join_cluster 'rabbit' 'rabbit.example.com'
+        salt '*' rabbitmq.join_cluster 'rabbit.example.com' 'rabbit'
     '''
+    if ram_node:
+        cmd = 'rabbitmqctl join_cluster --ram {0}@{1}'.format(user, host)
+    else:
+        cmd = 'rabbitmqctl join_cluster {0}@{1}'.format(user, host)
 
+    if runas is None:
+        runas = salt.utils.get_user()
     stop_app(runas)
-    res = __salt__['cmd.run'](
-        'rabbitmqctl join_cluster {0}@{1}'.format(user, host),
-        runas=runas)
+    res = __salt__['cmd.run'](cmd, runas=runas, python_shell=False)
     start_app(runas)
 
     return _format_response(res, 'Join')
@@ -360,6 +445,8 @@ def stop_app(runas=None):
 
         salt '*' rabbitmq.stop_app
     '''
+    if runas is None:
+        runas = salt.utils.get_user()
     res = __salt__['cmd.run'](
         'rabbitmqctl stop_app',
         runas=runas)
@@ -377,6 +464,8 @@ def start_app(runas=None):
 
         salt '*' rabbitmq.start_app
     '''
+    if runas is None:
+        runas = salt.utils.get_user()
     res = __salt__['cmd.run'](
         'rabbitmqctl start_app',
         runas=runas)
@@ -394,6 +483,8 @@ def reset(runas=None):
 
         salt '*' rabbitmq.reset
     '''
+    if runas is None:
+        runas = salt.utils.get_user()
     res = __salt__['cmd.run'](
         'rabbitmqctl reset',
         runas=runas)
@@ -411,6 +502,8 @@ def force_reset(runas=None):
 
         salt '*' rabbitmq.force_reset
     '''
+    if runas is None:
+        runas = salt.utils.get_user()
     res = __salt__['cmd.run'](
         'rabbitmqctl force_reset',
         runas=runas)
@@ -418,7 +511,7 @@ def force_reset(runas=None):
     return res
 
 
-def list_queues(*kwargs):
+def list_queues(runas=None, *kwargs):
     '''
     Returns queue details of the / virtual host
 
@@ -428,12 +521,17 @@ def list_queues(*kwargs):
 
         salt '*' rabbitmq.list_queues messages consumers
     '''
+    if runas is None:
+        runas = salt.utils.get_user()
     res = __salt__['cmd.run'](
-        'rabbitmqctl list_queues {0}'.format(' '.join(list(kwargs))))
+        'rabbitmqctl list_queues {0}'.format(' '.join(list(kwargs))),
+        python_shell=False,
+        runas=runas,
+        )
     return res
 
 
-def list_queues_vhost(vhost, *kwargs):
+def list_queues_vhost(vhost, runas=None, *kwargs):
     '''
     Returns queue details of specified virtual host. This command will consider
     first parameter as the vhost name and rest will be treated as
@@ -446,8 +544,16 @@ def list_queues_vhost(vhost, *kwargs):
 
         salt '*' rabbitmq.list_queues messages consumers
     '''
+    if runas is None:
+        runas = salt.utils.get_user()
     res = __salt__['cmd.run'](
-        'rabbitmqctl list_queues -p {0} {1}'.format(vhost, ' '.join(list(kwargs))))
+        'rabbitmqctl list_queues -p {0} {1}'.format(
+            vhost,
+            ' '.join(list(kwargs))
+            ),
+        python_shell=False,
+        runas=runas,
+        )
     return res
 
 
@@ -465,22 +571,28 @@ def list_policies(runas=None):
         salt '*' rabbitmq.list_policies'
     '''
     ret = {}
+    if runas is None:
+        runas = salt.utils.get_user()
     res = __salt__['cmd.run']('rabbitmqctl list_policies',
                               runas=runas)
     for line in res.splitlines():
         if '...' not in line and line != '\n':
             parts = line.split('\t')
-            if len(parts) != 6:
+            if len(parts) not in (5, 6):
                 continue
             vhost, name = parts[0], parts[1]
             if vhost not in ret:
                 ret[vhost] = {}
-            ret[vhost][name] = {
-                'apply_to': parts[2],
-                'pattern': parts[3],
-                'definition': parts[4],
-                'priority': parts[5]
-            }
+            ret[vhost][name] = {}
+            # How many fields are there? - 'apply_to' was inserted in position 2 at somepoint
+            offset = len(parts) - 5
+            if len(parts) == 6:
+                ret[vhost][name]['apply_to'] = parts[2]
+            ret[vhost][name].update({
+                'pattern': parts[offset+2],
+                'definition': parts[offset+3],
+                'priority': parts[offset+4]
+            })
     log.debug('Listing policies: {0}'.format(ret))
     return ret
 
@@ -497,6 +609,8 @@ def set_policy(vhost, name, pattern, definition, priority=None, runas=None):
 
         salt '*' rabbitmq.set_policy / HA '.*' '{"ha-mode": "all"}'
     '''
+    if runas is None:
+        runas = salt.utils.get_user()
     res = __salt__['cmd.run'](
         "rabbitmqctl set_policy -p {0}{1}{2} {3} '{4}' '{5}'".format(
             vhost,
@@ -505,6 +619,7 @@ def set_policy(vhost, name, pattern, definition, priority=None, runas=None):
             name,
             pattern,
             definition.replace("'", '"')),
+        python_shell=False,
         runas=runas)
     log.debug('Set policy: {0}'.format(res))
     return _format_response(res, 'Set')
@@ -522,9 +637,12 @@ def delete_policy(vhost, name, runas=None):
 
         salt '*' rabbitmq.delete_policy / HA'
     '''
+    if runas is None:
+        runas = salt.utils.get_user()
     res = __salt__['cmd.run'](
         'rabbitmqctl clear_policy -p {0} {1}'.format(
             vhost, name),
+        python_shell=False,
         runas=runas)
     log.debug('Delete policy: {0}'.format(res))
     return _format_response(res, 'Deleted')
@@ -542,11 +660,12 @@ def policy_exists(vhost, name, runas=None):
 
         salt '*' rabbitmq.policy_exists / HA
     '''
+    if runas is None:
+        runas = salt.utils.get_user()
     policies = list_policies(runas=runas)
     return bool(vhost in policies and name in policies[vhost])
 
 
-@decorators.which('rabbitmq-plugins')
 def plugin_is_enabled(name, runas=None):
     '''
     Return whether the plugin is enabled.
@@ -557,11 +676,14 @@ def plugin_is_enabled(name, runas=None):
 
         salt '*' rabbitmq.plugin_is_enabled foo
     '''
-    ret = __salt__['cmd.run']('rabbitmq-plugins list -m -e', runas=runas)
+    rabbitmq = _get_rabbitmq_plugin()
+    cmd = '{0} list -m -e'.format(rabbitmq)
+    if runas is None:
+        runas = salt.utils.get_user()
+    ret = __salt__['cmd.run'](cmd, python_shell=False, runas=runas)
     return bool(name in ret)
 
 
-@decorators.which('rabbitmq-plugins')
 def enable_plugin(name, runas=None):
     '''
     Enable a RabbitMQ plugin via the rabbitmq-plugins command.
@@ -572,13 +694,16 @@ def enable_plugin(name, runas=None):
 
         salt '*' rabbitmq.enable_plugin foo
     '''
-    ret = __salt__['cmd.run_all'](
-            'rabbitmq-plugins enable {0}'.format(name),
-            runas=runas)
+    rabbitmq = _get_rabbitmq_plugin()
+    cmd = '{0} enable {1}'.format(rabbitmq, name)
+
+    if runas is None:
+        runas = salt.utils.get_user()
+    ret = __salt__['cmd.run_all'](cmd, python_shell=False, runas=runas)
+
     return _format_response(ret, 'Enabled')
 
 
-@decorators.which('rabbitmq-plugins')
 def disable_plugin(name, runas=None):
     '''
     Disable a RabbitMQ plugin via the rabbitmq-plugins command.
@@ -590,7 +715,11 @@ def disable_plugin(name, runas=None):
         salt '*' rabbitmq.disable_plugin foo
     '''
 
-    ret = __salt__['cmd.run_all'](
-            'rabbitmq-plugins disable {0}'.format(name),
-            runas=runas)
+    rabbitmq = _get_rabbitmq_plugin()
+    cmd = '{0} disable {1}'.format(rabbitmq, name)
+
+    if runas is None:
+        runas = salt.utils.get_user()
+    ret = __salt__['cmd.run_all'](cmd, python_shell=False, runas=runas)
+
     return _format_response(ret, 'Disabled')
