@@ -2,6 +2,7 @@
 '''
 Module for running arbitrary tests
 '''
+from __future__ import absolute_import
 
 # Import Python libs
 import os
@@ -15,10 +16,69 @@ import random
 import salt
 import salt.version
 import salt.loader
+import salt.ext.six as six
+from salt.utils.decorators import depends
 
 __proxyenabled__ = ['*']
 
 
+@depends('non_existantmodulename')
+def missing_func():
+    return 'foo'
+
+
+def attr_call():
+    '''
+    Call grains.items via the attribute
+
+    CLI Example::
+
+    .. code-block:: bash
+
+        salt '*' test.attr_call
+    '''
+    return __salt__.grains.items()
+
+
+def module_report():
+    '''
+    Return a dict containing all of the exeution modules with a report on
+    the overall availability via different references
+
+    CLI Example::
+
+    .. code-block:: bash
+
+        salt '*' test.module_report
+    '''
+    ret = {'functions': [],
+           'function_attrs': [],
+           'function_subs': [],
+           'modules': [],
+           'module_attrs': [],
+           'missing_attrs': [],
+           'missing_subs': []}
+    for ref in __salt__:
+        if '.' in ref:
+            ret['functions'].append(ref)
+        else:
+            ret['modules'].append(ref)
+            if hasattr(__salt__, ref):
+                ret['module_attrs'].append(ref)
+            for func in __salt__[ref]:
+                full = '{0}.{1}'.format(ref, func)
+                if hasattr(getattr(__salt__, ref), func):
+                    ret['function_attrs'].append(full)
+                if func in __salt__[ref]:
+                    ret['function_subs'].append(full)
+    for func in ret['functions']:
+        if func not in ret['function_attrs']:
+            ret['missing_attrs'].append(func)
+        if func not in ret['function_subs']:
+            ret['missing_subs'].append(func)
+    return ret
+
+
 def echo(text):
     '''
     Return a string - used for testing the connection
@@ -91,7 +151,7 @@ def version():
 
         salt '*' test.version
     '''
-    return salt.__version__
+    return salt.version.__version__
 
 
 def versions_information():
@@ -212,7 +272,7 @@ def arg_type(*args, **kwargs):
         ret['args'].append(str(type(argument)))
 
     # all the kwargs
-    for key, val in kwargs.iteritems():
+    for key, val in six.iteritems(kwargs):
         ret['kwargs'][key] = str(type(val))
 
     return ret
@@ -360,8 +420,7 @@ def not_loaded():
     '''
     prov = providers()
     ret = set()
-    loader = salt.loader._create_loader(__opts__, 'modules', 'module')
-    for mod_dir in loader.module_dirs:
+    for mod_dir in salt.loader._module_dirs(__opts__, 'modules', 'module'):
         if not os.path.isabs(mod_dir):
             continue
         if not os.path.isdir(mod_dir):
@@ -447,3 +506,40 @@ def tty(*args, **kwargs):  # pylint: disable=W0613
         salt '*' test.tty pts3 'This is a test'
     '''
     return 'ERROR: This function has been moved to cmd.tty'
+
+
+def try_(module, return_try_exception=False, **kwargs):
+    '''
+    Try to run a module command. On an exception return None.
+    If `return_try_exception` is set True return the exception.
+    This can be helpfull in templates where running a module might fail as expected.
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        <pre>
+        {% for i in range(0,230) %}
+            {{ salt['test.try'](module='ipmi.get_users', bmc_host='172.2.2.'+i)|yaml(False) }}
+        {% endfor %}
+        </pre>
+    '''
+    try:
+        return __salt__[module](**kwargs)
+    except Exception as e:
+        if return_try_exception:
+            return e
+    return None
+
+
+def assertion(assertion):
+    '''
+    Assert the given argument
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' test.assert False
+    '''
+    assert assertion
