@@ -2,6 +2,7 @@
 '''
 Manage users with the useradd command
 '''
+from __future__ import absolute_import
 
 # Import python libs
 import re
@@ -15,7 +16,8 @@ import copy
 
 # Import salt libs
 import salt.utils
-from salt._compat import string_types
+from salt.ext.six import string_types
+from salt.exceptions import CommandExecutionError
 
 log = logging.getLogger(__name__)
 
@@ -29,7 +31,7 @@ __virtualname__ = 'user'
 
 def __virtual__():
     '''
-    Set the user module if the kernel is Linux or OpenBSD
+    Set the user module if the kernel is Linux, OpenBSD or NetBSD
     and remove some of the functionality on OS X
     '''
 
@@ -78,7 +80,8 @@ def add(name,
         roomnumber='',
         workphone='',
         homephone='',
-        createhome=True):
+        createhome=True,
+        loginclass=None):
     '''
     Add a user to the minion
 
@@ -96,25 +99,47 @@ def add(name,
     if gid not in (None, ''):
         cmd.extend(['-g', str(gid)])
     elif groups is not None and name in groups:
-        try:
-            for line in salt.utils.fopen('/etc/login.defs'):
-                if 'USERGROUPS_ENAB' not in line[:15]:
-                    continue
+        if __grains__['kernel'] != 'OpenBSD':
+            try:
+                for line in salt.utils.fopen('/etc/login.defs'):
+                    if 'USERGROUPS_ENAB' not in line[:15]:
+                        continue
 
-                if 'yes' in line:
-                    cmd.extend([
-                        '-g', str(__salt__['file.group_to_gid'](name))
-                    ])
+                    if 'yes' in line:
+                        cmd.extend([
+                            '-g', str(__salt__['file.group_to_gid'](name))
+                        ])
 
-                # We found what we wanted, let's break out of the loop
-                break
-        except OSError:
-            log.debug('Error reading /etc/login.defs', exc_info=True)
+                    # We found what we wanted, let's break out of the loop
+                    break
+            except OSError:
+                log.debug('Error reading /etc/login.defs', exc_info=True)
+        else:
+            try:
+                for line in salt.utils.fopen('/etc/usermgmt.conf'):
+                    if 'group' not in line[:5]:
+                        continue
 
-    if createhome:
-        cmd.append('-m')
-    elif createhome is False:
-        cmd.append('-M')
+                    for val in line.split(" "):
+                        cmd.extend([
+                            '-g', str(val[1])
+                        ])
+
+                    # We found what we wanted, let's break out of the loop
+                    break
+            except OSError:
+                # /etc/usermgmt.conf not present: defaults will be used
+                pass
+
+    if isinstance(createhome, bool):
+        if createhome:
+            cmd.append('-m')
+        elif (__grains__['kernel'] != 'NetBSD'
+                and __grains__['kernel'] != 'OpenBSD'):
+            cmd.append('-M')
+    else:
+        log.error('Value passes to ``createhome`` must be a boolean')
+        return False
 
     if home is not None:
         cmd.extend(['-d', home])
@@ -122,12 +147,18 @@ def add(name,
     if not unique:
         cmd.append('-o')
 
-    if system and __grains__['kernel'] != 'NetBSD':
+    if (system
+        and __grains__['kernel'] != 'NetBSD'
+        and __grains__['kernel'] != 'OpenBSD'):
         cmd.append('-r')
 
+    if __grains__['kernel'] == 'OpenBSD':
+        if loginclass is not None:
+            cmd.extend(['-L', loginclass])
+
     cmd.append(name)
 
-    ret = __salt__['cmd.run_all'](' '.join(cmd))
+    ret = __salt__['cmd.run_all'](cmd, python_shell=False)
 
     if ret['retcode'] != 0:
         return False
@@ -168,12 +199,12 @@ def delete(name, remove=False, force=False):
     if remove:
         cmd.append('-r')
 
-    if force:
+    if force and __grains__['kernel'] != 'OpenBSD':
         cmd.append('-f')
 
     cmd.append(name)
 
-    ret = __salt__['cmd.run_all'](' '.join(cmd))
+    ret = __salt__['cmd.run_all'](cmd, python_shell=False)
 
     if ret['retcode'] == 0:
         # Command executed with no errors
@@ -230,8 +261,8 @@ def chuid(name, uid):
     pre_info = info(name)
     if uid == pre_info['uid']:
         return True
-    cmd = 'usermod -u {0} {1}'.format(uid, name)
-    __salt__['cmd.run'](cmd)
+    cmd = ['usermod', '-u', '{0}'.format(uid), name]
+    __salt__['cmd.run'](cmd, python_shell=False)
     post_info = info(name)
     if post_info['uid'] != pre_info['uid']:
         return post_info['uid'] == uid
@@ -251,8 +282,8 @@ def chgid(name, gid):
     pre_info = info(name)
     if gid == pre_info['gid']:
         return True
-    cmd = 'usermod -g {0} {1}'.format(gid, name)
-    __salt__['cmd.run'](cmd)
+    cmd = ['usermod', '-g', '{0}'.format(gid), name]
+    __salt__['cmd.run'](cmd, python_shell=False)
     post_info = info(name)
     if post_info['gid'] != pre_info['gid']:
         return post_info['gid'] == gid
@@ -272,8 +303,8 @@ def chshell(name, shell):
     pre_info = info(name)
     if shell == pre_info['shell']:
         return True
-    cmd = 'usermod -s {0} {1}'.format(shell, name)
-    __salt__['cmd.run'](cmd)
+    cmd = ['usermod', '-s', shell, name]
+    __salt__['cmd.run'](cmd, python_shell=False)
     post_info = info(name)
     if post_info['shell'] != pre_info['shell']:
         return post_info['shell'] == shell
@@ -282,8 +313,8 @@ def chshell(name, shell):
 
 def chhome(name, home, persist=False):
     '''
-    Change the home directory of the user, pass true for persist to copy files
-    to the new home dir
+    Change the home directory of the user, pass True for persist to move files
+    to the new home directory if the old home directory exist.
 
     CLI Example:
 
@@ -295,10 +326,10 @@ def chhome(name, home, persist=False):
     if home == pre_info['home']:
         return True
     cmd = 'usermod -d {0} '.format(home)
-    if persist:
+    if persist and __grains__['kernel'] != 'OpenBSD':
         cmd += ' -m '
     cmd += name
-    __salt__['cmd.run'](cmd)
+    __salt__['cmd.run'](cmd, python_shell=False)
     post_info = info(name)
     if post_info['home'] != pre_info['home']:
         return post_info['home'] == home
@@ -322,10 +353,30 @@ def chgroups(name, groups, append=False):
     if ugrps == set(groups):
         return True
     cmd = 'usermod '
-    if append:
-        cmd += '-a '
-    cmd += '-G "{0}" {1}'.format(','.join(groups), name)
-    return not __salt__['cmd.retcode'](cmd)
+    if __grains__['kernel'] != 'OpenBSD':
+        if append:
+            cmd += '-a '
+    else:
+        if append:
+            cmd += '-G '
+        else:
+            cmd += '-S '
+    if __grains__['kernel'] != 'OpenBSD':
+        cmd += '-G '
+    cmd += '"{0}" {1}'.format(','.join(groups), name)
+    cmdret = __salt__['cmd.run_all'](cmd, python_shell=False)
+    ret = not cmdret['retcode']
+    # try to fallback on gpasswd to add user to localgroups
+    # for old lib-pamldap support
+    if __grains__['kernel'] != 'OpenBSD':
+        if not ret and ('not found in' in cmdret['stderr']):
+            ret = True
+            for group in groups:
+                cmd = 'gpasswd -a {0} {1}'.format(name, group)
+                cmdret = __salt__['cmd.run_all'](cmd, python_shell=False)
+                if cmdret['retcode']:
+                    ret = False
+    return ret
 
 
 def chfullname(name, fullname):
@@ -346,8 +397,8 @@ def chfullname(name, fullname):
         return True
     gecos_field = copy.deepcopy(pre_info)
     gecos_field['fullname'] = fullname
-    cmd = 'usermod -c "{0}" {1}'.format(_build_gecos(gecos_field), name)
-    __salt__['cmd.run'](cmd)
+    cmd = ['usermod', '-c', _build_gecos(gecos_field), name]
+    __salt__['cmd.run'](cmd, python_shell=False)
     post_info = info(name)
     if post_info['fullname'] != pre_info['fullname']:
         return post_info['fullname'] == fullname
@@ -372,8 +423,8 @@ def chroomnumber(name, roomnumber):
         return True
     gecos_field = copy.deepcopy(pre_info)
     gecos_field['roomnumber'] = roomnumber
-    cmd = 'usermod -c "{0}" {1}'.format(_build_gecos(gecos_field), name)
-    __salt__['cmd.run'](cmd)
+    cmd = ['usermod', '-c', _build_gecos(gecos_field), name]
+    __salt__['cmd.run'](cmd, python_shell=False)
     post_info = info(name)
     if post_info['roomnumber'] != pre_info['roomnumber']:
         return post_info['roomnumber'] == roomnumber
@@ -398,8 +449,8 @@ def chworkphone(name, workphone):
         return True
     gecos_field = copy.deepcopy(pre_info)
     gecos_field['workphone'] = workphone
-    cmd = 'usermod -c "{0}" {1}'.format(_build_gecos(gecos_field), name)
-    __salt__['cmd.run'](cmd)
+    cmd = ['usermod', '-c', _build_gecos(gecos_field), name]
+    __salt__['cmd.run'](cmd, python_shell=False)
     post_info = info(name)
     if post_info['workphone'] != pre_info['workphone']:
         return post_info['workphone'] == workphone
@@ -424,14 +475,37 @@ def chhomephone(name, homephone):
         return True
     gecos_field = copy.deepcopy(pre_info)
     gecos_field['homephone'] = homephone
-    cmd = 'usermod -c "{0}" {1}'.format(_build_gecos(gecos_field), name)
-    __salt__['cmd.run'](cmd)
+    cmd = ['usermod', '-c', _build_gecos(gecos_field), name]
+    __salt__['cmd.run'](cmd, python_shell=False)
     post_info = info(name)
     if post_info['homephone'] != pre_info['homephone']:
         return post_info['homephone'] == homephone
     return False
 
 
+def chloginclass(name, loginclass):
+    '''
+    Change the default login class of the user
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' user.chloginclass foo staff
+    '''
+    if __grains__['kernel'] != 'OpenBSD':
+        return False
+    pre_info = get_loginclass(name)
+    if loginclass == pre_info['loginclass']:
+        return True
+    cmd = 'usermod -L {0} {1}'.format(loginclass, name)
+    __salt__['cmd.run'](cmd)
+    post_info = get_loginclass(name)
+    if post_info['loginclass'] != pre_info['loginclass']:
+        return post_info['loginclass'] == loginclass
+    return False
+
+
 def info(name):
     '''
     Return user information
@@ -450,6 +524,29 @@ def info(name):
         return _format_info(data)
 
 
+def get_loginclass(name):
+    '''
+    Get the login class of the user
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' user.get_loginclass foo
+    '''
+    if __grains__['kernel'] != 'OpenBSD':
+        return False
+    userinfo = __salt__['cmd.run_stdout']('userinfo {0}'.format(name),
+        output_loglevel='debug')
+    for line in userinfo.splitlines():
+        if line.startswith("class"):
+            loginclass = line.split()
+    if len(loginclass) == 2:
+        return {'loginclass': loginclass[1]}
+    else:
+        return {'loginclass': '""'}
+
+
 def _format_info(data):
     '''
     Return user information in a pretty way
@@ -497,3 +594,27 @@ def list_users():
         salt '*' user.list_users
     '''
     return sorted([user.pw_name for user in pwd.getpwall()])
+
+
+def rename(name, new_name):
+    '''
+    Change the username for a named user
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' user.rename name new_name
+    '''
+    current_info = info(name)
+    if not current_info:
+        raise CommandExecutionError('User {0!r} does not exist'.format(name))
+    new_info = info(new_name)
+    if new_info:
+        raise CommandExecutionError('User {0!r} already exists'.format(new_name))
+    cmd = 'usermod -l {0} {1}'.format(new_name, name)
+    __salt__['cmd.run'](cmd)
+    post_info = info(new_name)
+    if post_info['name'] != current_info['name']:
+        return post_info['name'] == new_name
+    return False
