@@ -2,8 +2,15 @@
 '''
 Manage Windows features via the ServerManager powershell module
 '''
+from __future__ import absolute_import
 
 
+# Import python libs
+try:
+    from shlex import quote as _cmd_quote  # pylint: disable=E0611
+except ImportError:
+    from pipes import quote as _cmd_quote
+
 # Import salt libs
 import salt.utils
 
@@ -21,19 +28,28 @@ def _srvmgr(func):
     '''
     Execute a function from the ServerManager PS module and return the STDOUT
     '''
-    return __salt__['cmd.run']('Import-Module ServerManager ; {0}'.format(func), shell='powershell')
+    return __salt__['cmd.run'](
+            'Import-Module ServerManager ; {0}'.format(func),
+            shell='powershell',
+            python_shell=True)
 
 
 def _parse_powershell_list(lst):
     '''
     Parse command output when piped to format-list
+    Need to look at splitting with ':' so you can get the full value
+    Need to check for error codes and return false if it's trying to parse
     '''
     ret = {}
     for line in lst.splitlines():
         if line:
             splt = line.split()
-            ret[splt[0]] = splt[2]
-
+            # Ensure it's not a malformed line, e.g.:
+            #   FeatureResult : {foo, bar,
+            #                    baz}
+            if len(splt) > 2:
+                ret[splt[0]] = splt[2]
+    ret['message'] = lst
     return ret
 
 
@@ -47,7 +63,7 @@ def list_available():
 
         salt '*' win_servermanager.list_available
     '''
-    return _srvmgr('Get-WindowsFeature -erroraction silentlycontinue')
+    return _srvmgr('Get-WindowsFeature -erroraction silentlycontinue -warningaction silentlycontinue')
 
 
 def list_installed():
@@ -61,14 +77,17 @@ def list_installed():
         salt '*' win_servermanager.list_installed
     '''
     ret = {}
-    for line in list_available().splitlines()[2:]:
+    names = _srvmgr('Get-WindowsFeature -erroraction silentlycontinue -warningaction silentlycontinue | Select DisplayName,Name')
+    for line in names.splitlines()[2:]:
         splt = line.split()
-        if splt[0] == '[X]':
-            name = splt.pop(-1)
-            splt.pop(0)
-            display_name = ' '.join(splt)
-            ret[name] = display_name
-
+        name = splt.pop(-1)
+        display_name = ' '.join(splt)
+        ret[name] = display_name
+    state = _srvmgr('Get-WindowsFeature -erroraction silentlycontinue -warningaction silentlycontinue | Select InstallState,Name')
+    for line in state.splitlines()[2:]:
+        splt = line.split()
+        if splt[0] != 'Installed' and splt[1] in ret:
+            del ret[splt[1]]
     return ret
 
 
@@ -88,12 +107,13 @@ def install(feature, recurse=False):
     .. code-block:: bash
 
         salt '*' win_servermanager.install Telnet-Client
-        salt '*' win_servermanager.install SNMP-Services True
+        salt '*' win_servermanager.install SNMP-Service True
     '''
     sub = ''
     if recurse:
         sub = '-IncludeAllSubFeature'
-    out = _srvmgr('Add-WindowsFeature -Name {0} {1} -erroraction silentlycontinue | format-list'.format(feature, sub))
+    out = _srvmgr('Add-WindowsFeature -Name {0} {1} -erroraction silentlycontinue -warningaction silentlycontinue | format-list'.format(
+                  _cmd_quote(feature), sub))
     return _parse_powershell_list(out)
 
 
@@ -114,5 +134,6 @@ def remove(feature):
 
         salt -t 600 '*' win_servermanager.remove Telnet-Client
     '''
-    out = _srvmgr('Remove-WindowsFeature -Name {0} -erroraction silentlycontinue | format-list'.format(feature))
+    out = _srvmgr('Remove-WindowsFeature -Name {0} -erroraction silentlycontinue -warningaction silentlycontinue | format-list'.format(
+                  _cmd_quote(feature)))
     return _parse_powershell_list(out)
