@@ -2,6 +2,7 @@
 '''
 Execute a command and read the output as YAML. The YAML data is then directly overlaid onto the minion's Pillar data
 '''
+from __future__ import absolute_import
 
 # Don't "fix" the above docstring to put it on two lines, as the sphinx
 # autosummary pulls only the first line for its description.
@@ -16,7 +17,9 @@ import yaml
 log = logging.getLogger(__name__)
 
 
-def ext_pillar(minion_id, pillar, command):
+def ext_pillar(minion_id,  # pylint: disable=W0613
+               pillar,  # pylint: disable=W0613
+               command):
     '''
     Execute a command and read the output as YAML
     '''
