@@ -6,13 +6,14 @@ A flexible renderer that takes a templating engine and a data format
 :maturity: new
 :platform: all
 '''
+from __future__ import absolute_import
 # See http://docs.saltstack.org/en/latest/ref/renderers/all/salt.renderers.stateconf.html
 # for a guide to using this module.
 #
 # FIXME: I really need to review and simplify this renderer, it's getting out of hand!
 #
 # TODO:
-#   - sls meta/info state: Eg,
+#   - sls meta/info state: E.g.,
 #
 #       sls_info:
 #         stateconf.set:
@@ -20,7 +21,7 @@ A flexible renderer that takes a templating engine and a data format
 #           - description: what the salt file does...
 #           - version: 0.1.0
 #
-#   - version constraint for 'include'. Eg,
+#   - version constraint for 'include'. E.g.,
 #
 #       include:
 #         - apache: >= 0.1.0
@@ -37,6 +38,8 @@ from cStringIO import StringIO
 # Import salt libs
 import salt.utils
 from salt.exceptions import SaltRenderError
+import salt.ext.six as six
+from salt.ext.six import string_types
 
 __all__ = ['render']
 
@@ -45,7 +48,7 @@ log = logging.getLogger(__name__)
 
 __opts__ = {
     'stateconf_end_marker': r'#\s*-+\s*end of state config\s*-+',
-    # eg, something like "# --- end of state config --" works by default.
+    # e.g., something like "# --- end of state config --" works by default.
 
     'stateconf_start_state': '.start',
     # name of the state id for the generated start state.
@@ -185,7 +188,7 @@ def render(input, saltenv='base', sls='', argline='', **kws):
         ]
         try:
             name, rd_argline = (args[0] + ' ').split(' ', 1)
-            render_data = renderers[name]  # eg, the yaml renderer
+            render_data = renderers[name]  # e.g., the yaml renderer
             if implicit_require:
                 if name == 'yaml':
                     rd_argline = '-o ' + rd_argline
@@ -195,13 +198,13 @@ def render(input, saltenv='base', sls='', argline='', **kws):
                         'is used!'
                     )
             name, rt_argline = (args[1] + ' ').split(' ', 1)
-            render_template = renderers[name]  # eg, the mako renderer
+            render_template = renderers[name]  # e.g., the mako renderer
         except KeyError as err:
             raise SaltRenderError('Renderer: {0} is not available!'.format(err))
         except IndexError:
             raise INVALID_USAGE_ERROR
 
-        if isinstance(input, basestring):
+        if isinstance(input, string_types):
             with salt.utils.fopen(input, 'r') as ifile:
                 sls_templ = ifile.read()
         else:  # assume file-like
@@ -219,7 +222,7 @@ def render(input, saltenv='base', sls='', argline='', **kws):
             tmplctx = STATE_CONF.copy()
             if tmplctx:
                 prefix = sls + '::'
-                for k in tmplctx.keys():
+                for k in tmplctx.keys():  # iterate over a copy of keys
                     if k.startswith(prefix):
                         tmplctx[k[len(prefix):]] = tmplctx[k]
                         del tmplctx[k]
@@ -256,7 +259,7 @@ def rewrite_single_shorthand_state_decl(data):  # pylint: disable=C0103
         state.func: []
     '''
     for sid, states in data.items():
-        if isinstance(states, basestring):
+        if isinstance(states, string_types):
             data[sid] = {states: []}
 
 
@@ -309,7 +312,7 @@ def nvlist(thelist, names=None):
     for nvitem in thelist:
         if isinstance(nvitem, dict):
             # then nvitem is a name-value item(a dict) of the list.
-            name, value = nvitem.iteritems().next()
+            name, value = next(nvitem.iteritems())
             if names is None or name in names:
                 yield nvitem, name, value
 
@@ -335,12 +338,12 @@ def nvlist2(thelist, names=None):
 
 
 def statelist(states_dict, sid_excludes=frozenset(['include', 'exclude'])):
-    for sid, states in states_dict.iteritems():
+    for sid, states in six.iteritems(states_dict):
         if sid.startswith('__'):
             continue
         if sid in sid_excludes:
             continue
-        for sname, args in states.iteritems():
+        for sname, args in six.iteritems(states):
             if sname.startswith('__'):
                 continue
             yield sid, states, sname, args
@@ -366,7 +369,7 @@ def rename_state_ids(data, sls, is_extend=False):
             if sid.startswith('.'):
                 req[sname] = _local_to_abs_sid(sid, sls)
 
-    for sid in data.keys():
+    for sid in data:
         if sid.startswith('.'):
             newsid = _local_to_abs_sid(sid, sls)
             if newsid in data:
@@ -375,11 +378,11 @@ def rename_state_ids(data, sls, is_extend=False):
                     'already exists!'.format(sid, newsid)
                 )
             # add a '- name: sid' to those states without '- name'.
-            for sname, args in data[sid].iteritems():
+            for sname, args in six.iteritems(data[sid]):
                 if state_name(sname) == STATE_NAME:
                     continue
                 for arg in args:
-                    if isinstance(arg, dict) and iter(arg).next() == 'name':
+                    if isinstance(arg, dict) and next(iter(arg)) == 'name':
                         break
                 else:
                     # then no '- name: ...' is defined in the state args
@@ -452,7 +455,7 @@ def add_implicit_requires(data):
         if prev_state[0] is not None:
             try:
                 nvlist(args, ['require']).next()[2].insert(0, dict([prev_state]))
-            except StopIteration:  # ie, there's no require
+            except StopIteration:  # i.e., there's no require
                 args.append(dict(require=[dict([prev_state])]))
 
         states_before.add(tag)
@@ -473,14 +476,14 @@ def add_start_state(data, sls):
     # no __sls__, or it's the first state whose id declaration has a
     # __sls__ == sls.
     non_sids = set(['include', 'exclude', 'extend'])
-    for sid, states in data.iteritems():
+    for sid, states in six.iteritems(data):
         if sid in non_sids or sid.startswith('__'):
             continue
         if '__sls__' not in states or states['__sls__'] == sls:
             break
     else:
         raise SaltRenderError('Can\'t determine the first state in the sls file!')
-    reqin = {state_name(data[sid].iterkeys().next()): sid}
+    reqin = {state_name(next(data[sid].iterkeys())): sid}
     data[start_sid] = {STATE_FUNC: [{'require_in': [reqin]}]}
 
 
@@ -533,7 +536,7 @@ STATE_CONF_EXT = {}   # stateconf.set under extend: ...
 
 
 def extract_state_confs(data, is_extend=False):
-    for state_id, state_dict in data.iteritems():
+    for state_id, state_dict in six.iteritems(data):
         if state_id == 'extend' and not is_extend:
             extract_state_confs(state_dict, True)
             continue
@@ -550,7 +553,7 @@ def extract_state_confs(data, is_extend=False):
         for sdk in state_dict[key]:
             if not isinstance(sdk, dict):
                 continue
-            key, val = sdk.iteritems().next()
+            key, val = next(sdk.iteritems())
             conf[key] = val
 
         if not is_extend and state_id in STATE_CONF_EXT:
