@@ -14,14 +14,16 @@ authentication, it is also possible to pass private keys to use explicitly.
         - rev: develop
         - target: /tmp/salt
 '''
+from __future__ import absolute_import
 
 # Import python libs
 import logging
 import os
+import os.path
 import shutil
 
 # Import salt libs
-import salt.utils
+from salt.exceptions import CommandExecutionError
 
 log = logging.getLogger(__name__)
 
@@ -30,22 +32,25 @@ def __virtual__():
     '''
     Only load if git is available
     '''
-    return 'git' if __salt__['cmd.has_exec']('git') else False
+    return __salt__['cmd.has_exec']('git')
 
 
 def latest(name,
            rev=None,
            target=None,
-           runas=None,
            user=None,
            force=None,
            force_checkout=False,
+           force_reset=False,
            submodules=False,
            mirror=False,
            bare=False,
            remote_name='origin',
            always_fetch=False,
+           depth=None,
            identity=None,
+           https_user=None,
+           https_pass=None,
            onlyif=False,
            unless=False):
     '''
@@ -61,11 +66,6 @@ def latest(name,
     target
         Name of the target directory where repository is about to be cloned
 
-    runas
-        Name of the user performing repository management operations
-
-        .. deprecated:: 0.17.0
-
     user
         Name of the user performing repository management operations
 
@@ -78,6 +78,10 @@ def latest(name,
         Force a checkout even if there might be overwritten changes
         (Default: False)
 
+    force_reset
+        Force the checkout to ``--reset hard`` to the remote ref
+        (Default: False)
+
     submodules
         Update submodules on clone or branch change (Default: False)
 
@@ -99,8 +103,23 @@ def latest(name,
         until the tag or branch name changes. Setting this to true will force
         a fetch to occur. Only applies when rev is set. (Default: False)
 
+    depth
+        Defines depth in history when git a clone is needed in order to ensure
+        latest. E.g. ``depth: 1`` is usefull when deploying from a repository
+        with a long history. Use rev to specify branch. This is not compatible with tags or revision IDs.(Default: ``None``)
+
     identity
-        A path to a private key to use over SSH
+        A path on the minion server to a private key to use over SSH
+
+    https_user
+        HTTP Basic Auth username for HTTPS (only) clones
+
+        .. versionadded:: 2015.5.0
+
+    https_pass
+        HTTP Basic Auth password for HTTPS (only) clones
+
+        .. versionadded:: 2015.5.0
 
     onlyif
         A command to run as a check, run the named command only if the command
@@ -109,6 +128,44 @@ def latest(name,
     unless
         A command to run as a check, only run the named command if the command
         passed to the ``unless`` option returns false
+
+    .. note::
+
+        Clashing ID declarations can be avoided when including different
+        branches from the same git repository in the same sls file by using the
+        ``name`` declaration.  The example below checks out the ``gh-pages``
+        and ``gh-pages-prod`` branches from the same repository into separate
+        directories.  The example also sets up the ``ssh_known_hosts`` ssh key
+        required to perform the git checkout.
+
+    .. code-block:: yaml
+
+        gitlab.example.com:
+          ssh_known_hosts:
+            - present
+            - user: root
+            - enc: ecdsa
+            - fingerprint: 4e:94:b0:54:c1:5b:29:a2:70:0e:e1:a3:51:ee:ee:e3
+
+        git-website-staging:
+          git.latest:
+            - name: git@gitlab.example.com:user/website.git
+            - rev: gh-pages
+            - target: /usr/share/nginx/staging
+            - identity: /root/.ssh/website_id_rsa
+            - require:
+                - pkg: git
+                - ssh_known_hosts: gitlab.example.com
+
+        git-website-prod:
+          git.latest:
+            - name: git@gitlab.example.com:user/website.git
+            - rev: gh-pages-prod
+            - target: /usr/share/nginx/prod
+            - identity: /root/.ssh/website_id_rsa
+            - require:
+                - pkg: git
+                - ssh_known_hosts: gitlab.example.com
     '''
     ret = {'name': name, 'result': True, 'comment': '', 'changes': {}}
 
@@ -119,35 +176,14 @@ def latest(name,
 
     if not target:
         return _fail(ret, '"target" option is required')
-
-    salt.utils.warn_until(
-        'Lithium',
-        'Please remove \'runas\' support at this stage. \'user\' support was '
-        'added in 0.17.0',
-        _dont_call_warnings=True
-    )
-    if runas:
-        # Warn users about the deprecation
-        ret.setdefault('warnings', []).append(
-            'The \'runas\' argument is being deprecated in favor of \'user\', '
-            'please update your state files.'
-        )
-    if user is not None and runas is not None:
-        # user wins over runas but let warn about the deprecation.
-        ret.setdefault('warnings', []).append(
-            'Passed both the \'runas\' and \'user\' arguments. Please don\'t. '
-            '\'runas\' is being ignored in favor of \'user\'.'
-        )
-        runas = None
-    elif runas is not None:
-        # Support old runas usage
-        user = runas
-        runas = None
+    target = os.path.expanduser(target)
 
     run_check_cmd_kwargs = {'runas': user}
+    if 'shell' in __grains__:
+        run_check_cmd_kwargs['shell'] = __grains__['shell']
 
     # check if git.latest should be applied
-    cret = _run_check(
+    cret = mod_run_check(
         run_check_cmd_kwargs, onlyif, unless
     )
     if isinstance(cret, dict):
@@ -156,29 +192,38 @@ def latest(name,
 
     bare = bare or mirror
     check = 'refs' if bare else '.git'
-
-    if os.path.isdir(target) and os.path.isdir('{0}/{1}'.format(target,
-                                                                check)):
+    checkdir = os.path.join(target, check)
+    if os.path.isdir(target) and os.path.isdir(checkdir):
         # git pull is probably required
         log.debug(('target {0} is found, "git pull" '
                    'is probably required'.format(target)))
         try:
-            current_rev = __salt__['git.revision'](target, user=user)
+            try:
+                current_rev = __salt__['git.revision'](target, user=user)
+            except CommandExecutionError:
+                current_rev = None
 
             # handle the case where a branch was provided for rev
             remote_rev, new_rev = None, None
-            branch = __salt__['git.current_branch'](target, user=user)
+            try:
+                branch = __salt__['git.current_branch'](target, user=user)
+            except CommandExecutionError:
+                branch = None
+
             # We're only interested in the remote branch if a branch
             # (instead of a hash, for example) was provided for rev.
-            if len(branch) > 0 and branch == rev:
+            if (branch != 'HEAD' and branch == rev) or rev is None:
                 remote_rev = __salt__['git.ls_remote'](target,
                                                        repository=name,
-                                                       branch=branch, user=user,
-                                                       identity=identity)
+                                                       branch=branch,
+                                                       user=user,
+                                                       identity=identity,
+                                                       https_user=https_user,
+                                                       https_pass=https_pass)
 
             # only do something, if the specified rev differs from the
             # current_rev and remote_rev
-            if current_rev in [rev, remote_rev]:
+            if current_rev in [rev, remote_rev] or (remote_rev is not None and remote_rev.startswith(current_rev)):
                 new_rev = current_rev
             else:
 
@@ -203,8 +248,15 @@ def latest(name,
                     __salt__['git.remote_set'](target,
                                                name=remote_name,
                                                url=name,
-                                               user=user)
-                    ret['changes']['remote/{0}'.format(remote_name)] = "{0} => {1}".format(str(remote), name)
+                                               user=user,
+                                               https_user=https_user,
+                                               https_pass=https_pass)
+                    ret['changes']['remote/{0}'.format(remote_name)] = (
+                        "{0} => {1}".format(str(remote), name)
+                    )
+                    # Set to fetch later since we just added the remote and
+                    # need to get the refs
+                    always_fetch = True
 
                 # check if rev is already present in repo, git-fetch otherwise
                 if bare:
@@ -225,19 +277,54 @@ def latest(name,
                                               user=user,
                                               identity=identity)
 
+                    if force_reset:
+                        opts = "--hard {0}/{1}".format(remote_name, rev)
+                        __salt__['git.reset'](target,
+                                              opts=opts,
+                                              user=user)
+
                     __salt__['git.checkout'](target,
                                              rev,
                                              force=force_checkout,
                                              user=user)
 
+                    if branch != 'HEAD':
+                        try:
+                            current_remote = __salt__['git.config_get'](
+                                target, 'branch.{0}.remote'.format(rev),
+                                user=user)
+                        except CommandExecutionError:
+                            current_remote = None
+
+                        if current_remote != remote_name:
+                            if __opts__['test']:
+                                ret['changes'] = {'old': current_remote,
+                                                  'new': remote_name}
+                                return _neutral_test(
+                                    ret,
+                                    ('Repository {0} update is probably '
+                                     'required. Current remote is {1} should '
+                                     'be {2}'.format(target, current_remote,
+                                                     remote_name)))
+                            log.debug(
+                                'Setting branch {0} to '
+                                'upstream {1}'.format(rev, remote_name))
+                            __salt__['git.branch'](
+                                target, rev,
+                                opts='--set-upstream '
+                                '{0}/{1}'.format(remote_name, rev),
+                                user=user)
+                            ret['changes']['remote/''{0}/{1}'.format(
+                                remote_name, rev)] = (
+                                    '{0} => {1}'.format(current_remote,
+                                                        remote_name))
+
                 # check if we are on a branch to merge changes
-                cmd = "git symbolic-ref -q HEAD > /dev/null"
+                cmd = "git symbolic-ref -q HEAD"
                 retcode = __salt__['cmd.retcode'](cmd, cwd=target, runas=user)
                 if 0 == retcode:
-                    __salt__['git.fetch' if bare else 'git.pull'](target,
-                                                                  opts=fetch_opts,
-                                                                  user=user,
-                                                                  identity=identity)
+                    __salt__['git.fetch' if bare else 'git.pull'](
+                        target, opts=fetch_opts, user=user, identity=identity)
 
                 if submodules:
                     __salt__['git.submodule'](target,
@@ -245,11 +332,15 @@ def latest(name,
                                               identity=identity,
                                               opts='--recursive')
 
-                new_rev = __salt__['git.revision'](cwd=target, user=user)
+                try:
+                    new_rev = __salt__['git.revision'](cwd=target, user=user)
+                except CommandExecutionError:
+                    new_rev = None
         except Exception as exc:
+            log.error('Unexpected exception in git state', exc_info=True)
             return _fail(
-                    ret,
-                    str(exc))
+                ret,
+                str(exc))
 
         if current_rev != new_rev:
             log.info('Repository {0} updated: {1} => {2}'.format(target,
@@ -257,7 +348,7 @@ def latest(name,
                                                                  new_rev))
             ret['comment'] = 'Repository {0} updated'.format(target)
             ret['changes']['revision'] = '{0} => {1}'.format(
-                    current_rev, new_rev)
+                current_rev, new_rev)
     else:
         if os.path.isdir(target):
             # git clone is required, target exists but force is turned on
@@ -270,31 +361,40 @@ def latest(name,
                     shutil.rmtree(target)
             # git clone is required, but target exists and is non-empty
             elif os.listdir(target):
-                return _fail(ret, 'Directory \'{0}\' exists, is non-empty, and '
-                             'force option not in use'.format(target))
+                return _fail(ret, 'Directory \'{0}\' exists, is non-empty, '
+                             'and force option not in use'.format(target))
 
         # git clone is required
         log.debug(
-                'target {0} is not found, "git clone" is required'.format(
-                    target))
+            'target {0} is not found, "git clone" is required'.format(
+                target))
         if 'test' in __opts__:
             if __opts__['test']:
                 return _neutral_test(
-                        ret,
-                        'Repository {0} is about to be cloned to {1}'.format(
-                            name, target))
+                    ret,
+                    'Repository {0} is about to be cloned to {1}'.format(
+                        name, target))
         try:
             # make the clone
             opts = '--mirror' if mirror else '--bare' if bare else ''
             # if remote_name is not origin add --origin <name> to opts
             if remote_name != 'origin':
                 opts += ' --origin {0}'.format(remote_name)
+
+            # if depth is given add --depth <depth> to opts
+            if depth is not None:
+                opts += ' --depth {0}'.format(depth)
+                if rev is not None:
+                    opts += ' --branch {0}'.format(rev)
+
             # do the clone
             __salt__['git.clone'](target,
                                   name,
                                   user=user,
                                   opts=opts,
-                                  identity=identity)
+                                  identity=identity,
+                                  https_user=https_user,
+                                  https_pass=https_pass)
 
             if rev and not bare:
                 __salt__['git.checkout'](target, rev, user=user)
@@ -305,13 +405,18 @@ def latest(name,
                                           identity=identity,
                                           opts='--recursive')
 
-            new_rev = None if bare else (
-                __salt__['git.revision'](cwd=target, user=user))
+            new_rev = None
+            if not bare:
+                try:
+                    new_rev = __salt__['git.revision'](cwd=target, user=user)
+                except CommandExecutionError:
+                    pass
 
         except Exception as exc:
+            log.error('Unexpected exception in git state', exc_info=True)
             return _fail(
-                    ret,
-                    str(exc))
+                ret,
+                str(exc))
 
         message = 'Repository {0} cloned to {1}'.format(name, target)
         log.info(message)
@@ -322,7 +427,7 @@ def latest(name,
     return ret
 
 
-def present(name, bare=True, runas=None, user=None, force=False):
+def present(name, bare=True, user=None, force=False, shared=None):
     '''
     Make sure the repository is present in the given directory
 
@@ -332,11 +437,6 @@ def present(name, bare=True, runas=None, user=None, force=False):
     bare
         Create a bare repository (Default: True)
 
-    runas
-        Name of the user performing repository management operations
-
-        .. deprecated:: 0.17.0
-
     user
         Name of the user performing repository management operations
 
@@ -345,38 +445,20 @@ def present(name, bare=True, runas=None, user=None, force=False):
     force
         Force-create a new repository into an pre-existing non-git directory
         (deletes contents)
-    '''
-    ret = {'name': name, 'result': True, 'comment': '', 'changes': {}}
 
-    salt.utils.warn_until(
-        'Lithium',
-        'Please remove \'runas\' support at this stage. \'user\' support was '
-        'added in 0.17.0',
-        _dont_call_warnings=True
-    )
-    if runas:
-        # Warn users about the deprecation
-        ret.setdefault('warnings', []).append(
-            'The \'runas\' argument is being deprecated in favor of \'user\', '
-            'please update your state files.'
-        )
-    if user is not None and runas is not None:
-        # user wins over runas but let warn about the deprecation.
-        ret.setdefault('warnings', []).append(
-            'Passed both the \'runas\' and \'user\' arguments. Please don\'t. '
-            '\'runas\' is being ignored in favor of \'user\'.'
-        )
-        runas = None
-    elif runas is not None:
-        # Support old runas usage
-        user = runas
-        runas = None
+    shared
+        Specify the permission for sharing, see git-init for details (Default: None)
+
+       .. versionadded:: XXXX
+    '''
+    name = os.path.expanduser(name)
+    ret = {'name': name, 'result': True, 'comment': '', 'changes': {}}
 
     # If the named directory is a git repo return True
     if os.path.isdir(name):
-        if bare and os.path.isfile('{0}/HEAD'.format(name)):
+        if bare and os.path.isfile(os.path.join(name, 'HEAD')):
             return ret
-        elif not bare and os.path.isdir('{0}/.git'.format(name)):
+        elif not bare and os.path.isdir(os.path.join(name, '.git')):
             return ret
         # Directory exists and is not a git repo, if force is set destroy the
         # directory and recreate, otherwise throw an error
@@ -400,6 +482,7 @@ def present(name, bare=True, runas=None, user=None, force=False):
             shutil.rmtree(name)
 
     opts = '--bare' if bare else ''
+    opts += ' --shared={0}'.format(shared) if shared else ''
     __salt__['git.init'](cwd=name, user=user, opts=opts)
 
     message = 'Initialized repository {0}'.format(name)
@@ -410,6 +493,91 @@ def present(name, bare=True, runas=None, user=None, force=False):
     return ret
 
 
+def config(name,
+           value,
+           repo=None,
+           user=None,
+           is_global=False):
+    '''
+    .. versionadded:: 2014.7.0
+
+    Manage a git config setting for a user or repository
+
+    name
+        Name of the git config value to set
+
+    value
+        Value to set
+
+    repo : None
+        An optional location of a git repository for local operations
+
+    user : None
+        Optional name of a user as whom `git config` will be run
+
+    is_global : False
+        Whether or not to pass the `--global` option to `git config`
+
+    Local config example:
+
+    .. code-block:: yaml
+
+        mylocalrepo:
+          git.config:
+            - name: user.email
+            - value: fester@bestertester.net
+            - repo: file://my/path/to/repo
+
+    Global config example:
+
+    .. code-block:: yaml
+
+        mylocalrepo:
+          git.config:
+            - name: user.name
+            - value: Esther Bestertester
+            - user: ebestertester
+            - is_global: True
+    '''
+    ret = {'name': name,
+           'changes': {},
+           'result': True,
+           'comment': ''}
+
+    # get old value
+    try:
+        oval = __salt__['git.config_get'](setting_name=name,
+                                          cwd=repo,
+                                          user=user)
+    except CommandExecutionError:
+        oval = None
+
+    if value == oval:
+        ret['comment'] = 'No changes made'
+    else:
+        if __opts__['test']:
+            nval = value
+        else:
+            # set new value
+            __salt__['git.config_set'](setting_name=name,
+                                       setting_value=value,
+                                       cwd=repo,
+                                       user=user,
+                                       is_global=is_global)
+
+            # get new value
+            nval = __salt__['git.config_get'](setting_name=name,
+                                              cwd=repo,
+                                              user=user)
+
+        if oval is None:
+            oval = 'None'
+
+        ret['changes'][name] = '{0} => {1}'.format(oval, nval)
+
+    return ret
+
+
 def _fail(ret, comment):
     ret['result'] = False
     ret['comment'] = comment
@@ -422,7 +590,7 @@ def _neutral_test(ret, comment):
     return ret
 
 
-def _run_check(cmd_kwargs, onlyif, unless):
+def mod_run_check(cmd_kwargs, onlyif, unless):
     '''
     Execute the onlyif and unless logic.
     Return a result dict if:
@@ -433,11 +601,13 @@ def _run_check(cmd_kwargs, onlyif, unless):
     if onlyif:
         if __salt__['cmd.retcode'](onlyif, **cmd_kwargs) != 0:
             return {'comment': 'onlyif execution failed',
+                    'skip_watch': True,
                     'result': True}
 
     if unless:
         if __salt__['cmd.retcode'](unless, **cmd_kwargs) == 0:
             return {'comment': 'unless execution succeeded',
+                    'skip_watch': True,
                     'result': True}
 
     # No reason to stop, return True
