@@ -2,6 +2,7 @@
 '''
 Print a stacktrace when sent a SIGUSR1 for debugging
 '''
+from __future__ import absolute_import
 
 # Import python libs
 import os
@@ -42,6 +43,28 @@ def _handle_sigusr1(sig, stack):
             _makepretty(output, stack)
 
 
+def _handle_sigusr2(sig, stack):
+    '''
+    Signal handler for SIGUSR2, only available on Unix-like systems
+    '''
+    try:
+        import yappi
+    except ImportError:
+        return
+    if yappi.is_running():
+        yappi.stop()
+        filename = 'callgrind.salt-{0}-{1}'.format(int(time.time()), os.getpid())
+        destfile = os.path.join(tempfile.gettempdir(), filename)
+        yappi.get_func_stats().save(destfile, type='CALLGRIND')
+        if sys.stderr.isatty():
+            sys.stderr.write('Saved profiling data to: {0}\n'.format(destfile))
+        yappi.clear_stats()
+    else:
+        if sys.stderr.isatty():
+            sys.stderr.write('Profiling started\n')
+        yappi.start()
+
+
 def enable_sig_handler(signal_name, handler):
     '''
     Add signal handler for signal name if it exists on given platform
@@ -56,11 +79,18 @@ def enable_sigusr1_handler():
     when any of the salt daemons such as salt-master are sent a SIGUSR1
     '''
     enable_sig_handler('SIGUSR1', _handle_sigusr1)
-    # Also canonical BSD-way of printing profress is SIGINFO
-    # which on BSD-deriviatives can be sent via Ctrl+T
+    # Also canonical BSD-way of printing progress is SIGINFO
+    # which on BSD-derivatives can be sent via Ctrl+T
     enable_sig_handler('SIGINFO', _handle_sigusr1)
 
 
+def enable_sigusr2_handler():
+    '''
+    Toggle YAPPI profiler
+    '''
+    enable_sig_handler('SIGUSR2', _handle_sigusr2)
+
+
 def inspect_stack():
     '''
     Return a string of which function we are currently in.
