@@ -18,7 +18,7 @@ class ArgumentTestCase(integration.ModuleCase):
         '''
         Test passing a non-supported keyword argument. The relevant code that
         checks for invalid kwargs is located in salt/minion.py, within the
-        'parse_args_and_kwargs' function.
+        'load_args_and_kwargs' function.
         '''
         self.assertIn(
             ("ERROR executing 'test.ping': The following keyword arguments"),
