@@ -3,11 +3,14 @@
 '''
 Discover all instances of unittest.TestCase in this directory.
 '''
+# pylint: disable=file-perms
+
 # Import python libs
 from __future__ import print_function
 import os
 import resource
 import tempfile
+import time
 
 # Import salt libs
 from integration import TestDaemon, TMP  # pylint: disable=W0403
@@ -49,12 +52,17 @@ class SaltTestsuiteParser(SaltCoverageTestingParser):
             action='store_true',
             help='Print some system information.'
         )
-        # Just so it does not throw an error
         self.add_option(
             '--transport',
             default='zeromq',
-            dest='transport',
-            help='Not applicable in 2014.x.x')
+            choices=('zeromq', 'raet'),
+            help='Set to raet to run integration tests with raet transport. Default: %default')
+        self.add_option(
+            '--interactive',
+            default=False,
+            action='store_true',
+            help='Do not run any tests. Simply start the daemons.'
+        )
 
         self.test_selection_group.add_option(
             '-m',
@@ -81,7 +89,8 @@ class SaltTestsuiteParser(SaltCoverageTestingParser):
             dest='cli',
             default=False,
             action='store_true',
-            help='Run tests for cli')
+            help='Run tests for cli'
+        )
         self.test_selection_group.add_option(
             '-c',
             '--client',
@@ -101,11 +110,11 @@ class SaltTestsuiteParser(SaltCoverageTestingParser):
         )
         self.test_selection_group.add_option(
             '-r',
-            '--runner',
-            dest='runner',
+            '--runners',
+            dest='runners',
             default=False,
             action='store_true',
-            help='Run runner tests'
+            help='Run salt/runners/*.py tests'
         )
         self.test_selection_group.add_option(
             '-l',
@@ -124,13 +133,50 @@ class SaltTestsuiteParser(SaltCoverageTestingParser):
             help='Run unit tests'
         )
         self.test_selection_group.add_option(
-            '-W',
-            '--wheel',
+            '--fileserver-tests',
+            dest='fileserver',
             default=False,
             action='store_true',
+            help='Run Fileserver tests'
+        )
+        self.test_selection_group.add_option(
+            '-w',
+            '--wheel',
+            action='store_true',
+            default=False,
             help='Run wheel tests'
         )
-
+        self.test_selection_group.add_option(
+            '-o',
+            '--outputter',
+            action='store_true',
+            default=False,
+            help='Run outputter tests'
+        )
+        self.test_selection_group.add_option(
+            '--cloud-provider-tests',
+            action='store_true',
+            default=False,
+            help=('Run cloud provider tests. These tests create and delete '
+                  'instances on cloud providers. Must provide valid credentials '
+                  'in salt/tests/integration/files/conf/cloud.*.d to run tests.')
+        )
+        self.test_selection_group.add_option(
+            '--ssh',
+            action='store_true',
+            default=False,
+            help='Run salt-ssh tests. These tests will spin up a temporary '
+                 'SSH server on your machine. In certain environments, this '
+                 'may be insecure! Default: False'
+        )
+        self.test_selection_group.add_option(
+            '-A',
+            '--api-tests',
+            dest='api',
+            action='store_true',
+            default=False,
+            help='Run salt-api tests'
+        )
         self.output_options_group.add_option(
             '--no-colors',
             '--no-colours',
@@ -140,12 +186,26 @@ class SaltTestsuiteParser(SaltCoverageTestingParser):
         )
 
     def validate_options(self):
+        if self.options.cloud_provider_tests:
+            # Turn on expensive tests execution
+            os.environ['EXPENSIVE_TESTS'] = 'True'
+
         if self.options.coverage and any((
-                self.options.module, self.options.cli, self.options.client,
-                self.options.shell, self.options.unit, self.options.state,
-                self.options.runner, self.options.loader, self.options.name,
+                self.options.module,
+                self.options.cli,
+                self.options.client,
+                self.options.shell,
+                self.options.unit,
+                self.options.state,
+                self.options.runners,
+                self.options.loader,
+                self.options.name,
+                self.options.outputter,
+                self.options.fileserver,
                 self.options.wheel,
-                os.geteuid() != 0, not self.options.run_destructive)):
+                self.options.api,
+                os.geteuid() != 0,
+                not self.options.run_destructive)):
             self.error(
                 'No sense in generating the tests coverage report when '
                 'not running the full test suite, including the '
@@ -154,25 +214,37 @@ class SaltTestsuiteParser(SaltCoverageTestingParser):
             )
 
         # Set test suite defaults if no specific suite options are provided
-        if not any((self.options.module, self.options.cli, self.options.client,
+        if not any((self.options.module, self.options.client, self.options.cli,
                     self.options.shell, self.options.unit, self.options.state,
-                    self.options.runner, self.options.loader,
-                    self.options.name, self.options.wheel)):
+                    self.options.runners, self.options.loader, self.options.name,
+                    self.options.outputter, self.options.cloud_provider_tests,
+                    self.options.fileserver, self.options.wheel, self.options.api)):
             self.options.module = True
             self.options.cli = True
             self.options.client = True
             self.options.shell = True
             self.options.unit = True
-            self.options.runner = True
+            self.options.runners = True
             self.options.state = True
             self.options.loader = True
+            self.options.outputter = True
+            self.options.fileserver = True
             self.options.wheel = True
+            self.options.api = True
 
         self.start_coverage(
             branch=True,
             source=[os.path.join(SALT_ROOT, 'salt')],
         )
 
+        # Transplant configuration
+        TestDaemon.transplant_configs(transport=self.options.transport)
+
+    def post_execution_cleanup(self):
+        SaltCoverageTestingParser.post_execution_cleanup(self)
+        if self.options.clean:
+            TestDaemon.clean()
+
     def run_integration_suite(self, suite_folder, display_name):
         '''
         Run an integration test suite
@@ -180,35 +252,51 @@ class SaltTestsuiteParser(SaltCoverageTestingParser):
         path = os.path.join(TEST_DIR, 'integration', suite_folder)
         return self.run_suite(path, display_name)
 
-    def run_integration_tests(self):
-        '''
-        Execute the integration tests suite
-        '''
-        named_tests = []
-        named_unit_test = []
+    def start_daemons_only(self):
+        self.prep_filehandles()
+        try:
+            print_header(
+                ' * Setting up Salt daemons for interactive use',
+                top=False, width=getattr(self.options, 'output_columns', PNUM)
+            )
+        except TypeError:
+            print_header(' * Setting up Salt daemons for interactive use', top=False)
 
-        if self.options.name:
-            for test in self.options.name:
-                if test.startswith('unit.'):
-                    named_unit_test.append(test)
-                    continue
-                named_tests.append(test)
+        with TestDaemon(self):
+            print_header(' * Salt daemons started')
+            master_conf = TestDaemon.config('master')
+            minion_conf = TestDaemon.config('minion')
+            syndic_conf = TestDaemon.config('syndic')
+            syndic_master_conf = TestDaemon.config('syndic_master')
 
-        if (self.options.unit or named_unit_test) and not \
-                (self.options.runner or
-                 self.options.state or
-                 self.options.module or
-                 self.options.cli or
-                 self.options.client or
-                 self.options.loader or
-                 self.options.wheel or
-                 named_tests):
-            # We're either not running any of runner, state, module and client
-            # tests, or, we're only running unittests by passing --unit or by
-            # passing only `unit.<whatever>` to --name.
-            # We don't need the tests daemon running
-            return [True]
+            print_header(' * Syndic master configuration values', top=False)
+            print('interface: {0}'.format(syndic_master_conf['interface']))
+            print('publish port: {0}'.format(syndic_master_conf['publish_port']))
+            print('return port: {0}'.format(syndic_master_conf['ret_port']))
+            print('\n')
 
+            print_header(' * Master configuration values', top=True)
+            print('interface: {0}'.format(master_conf['interface']))
+            print('publish port: {0}'.format(master_conf['publish_port']))
+            print('return port: {0}'.format(master_conf['ret_port']))
+            print('\n')
+
+            print_header(' * Minion configuration values', top=True)
+            print('interface: {0}'.format(minion_conf['interface']))
+            print('\n')
+
+            print_header(' * Syndic configuration values', top=True)
+            print('interface: {0}'.format(syndic_conf['interface']))
+            print('syndic master port: {0}'.format(syndic_conf['syndic_master']))
+            print('\n')
+
+            print_header(' Your client configuration is at {0}'.format(TestDaemon.config_location()))
+            print('To access the minion: `salt -c {0} minion test.ping'.format(TestDaemon.config_location()))
+
+            while True:
+                time.sleep(1)
+
+    def prep_filehandles(self):
         smax_open_files, hmax_open_files = resource.getrlimit(
             resource.RLIMIT_NOFILE
         )
@@ -236,15 +324,56 @@ class SaltTestsuiteParser(SaltCoverageTestingParser):
                 print('  ulimit -n {0}'.format(REQUIRED_OPEN_FILES))
                 self.exit()
             finally:
-                print('~' * PNUM)
+                print('~' * getattr(self.options, 'output_columns', PNUM))
+
+    def run_integration_tests(self):
+        '''
+        Execute the integration tests suite
+        '''
+        named_tests = []
+        named_unit_test = []
+
+        if self.options.name:
+            for test in self.options.name:
+                if test.startswith('unit.'):
+                    named_unit_test.append(test)
+                    continue
+                named_tests.append(test)
+
+        if (self.options.unit or named_unit_test) and not \
+                (self.options.runners or
+                 self.options.state or
+                 self.options.module or
+                 self.options.cli or
+                 self.options.client or
+                 self.options.loader or
+                 self.options.outputter or
+                 self.options.fileserver or
+                 self.options.wheel or
+                 self.options.cloud_provider_tests or
+                 self.options.api or
+                 named_tests):
+            # We're either not running any of runners, state, module and client
+            # tests, or, we're only running unittests by passing --unit or by
+            # passing only `unit.<whatever>` to --name.
+            # We don't need the tests daemon running
+            return [True]
+        self.prep_filehandles()
+
+        try:
+            print_header(
+                ' * Setting up Salt daemons to execute tests',
+                top=False, width=getattr(self.options, 'output_columns', PNUM)
+            )
+        except TypeError:
+            print_header(' * Setting up Salt daemons to execute tests', top=False)
 
-        print_header('Setting up Salt daemons to execute tests', top=False)
         status = []
-        if not any([self.options.cli, self.options.client,
-                    self.options.module, self.options.runner,
-                    self.options.shell, self.options.state,
-                    self.options.loader, self.options.name,
-                    self.options.wheel]):
+        if not any([self.options.cli, self.options.client, self.options.module,
+                    self.options.runners, self.options.shell, self.options.state,
+                    self.options.loader, self.options.outputter, self.options.name,
+                    self.options.cloud_provider_tests, self.options.api,
+                    self.options.fileserver, self.options.wheel]):
             return status
 
         with TestDaemon(self):
@@ -256,8 +385,8 @@ class SaltTestsuiteParser(SaltCoverageTestingParser):
                     status.append(results)
             if self.options.loader:
                 status.append(self.run_integration_suite('loader', 'Loader'))
-            if self.options.runner:
-                status.append(self.run_integration_suite('runners', 'Runner'))
+            if self.options.runners:
+                status.append(self.run_integration_suite('runners', 'Runners'))
             if self.options.module:
                 status.append(self.run_integration_suite('modules', 'Module'))
             if self.options.state:
@@ -268,8 +397,16 @@ class SaltTestsuiteParser(SaltCoverageTestingParser):
                 status.append(self.run_integration_suite('client', 'Client'))
             if self.options.shell:
                 status.append(self.run_integration_suite('shell', 'Shell'))
+            if self.options.outputter:
+                status.append(self.run_integration_suite('output', 'Outputter'))
+            if self.options.fileserver:
+                status.append(self.run_integration_suite('fileserver', 'Fileserver'))
             if self.options.wheel:
                 status.append(self.run_integration_suite('wheel', 'Wheel'))
+            if self.options.cloud_provider_tests:
+                status.append(self.run_integration_suite('cloud/providers', 'Cloud Provider'))
+            if self.options.api:
+                status.append(self.run_integration_suite('netapi', 'NetAPI'))
         return status
 
     def run_unit_tests(self):
@@ -310,23 +447,29 @@ def main():
     '''
     Parse command line options for running specific tests
     '''
-    parser = SaltTestsuiteParser(
-        TEST_DIR,
-        xml_output_dir=XML_OUTPUT_DIR,
-        tests_logfile=os.path.join(tempfile.gettempdir(), 'salt-runtests.log')
-    )
-    parser.parse_args()
+    try:
+        parser = SaltTestsuiteParser(
+            TEST_DIR,
+            xml_output_dir=XML_OUTPUT_DIR,
+            tests_logfile=os.path.join(tempfile.gettempdir(), 'salt-runtests.log')
+        )
+        parser.parse_args()
 
-    overall_status = []
-    status = parser.run_integration_tests()
-    overall_status.extend(status)
-    status = parser.run_unit_tests()
-    overall_status.extend(status)
-    false_count = overall_status.count(False)
+        overall_status = []
+        if parser.options.interactive:
+            parser.start_daemons_only()
+        status = parser.run_integration_tests()
+        overall_status.extend(status)
+        status = parser.run_unit_tests()
+        overall_status.extend(status)
+        false_count = overall_status.count(False)
 
-    if false_count > 0:
-        parser.finalize(1)
-    parser.finalize(0)
+        if false_count > 0:
+            parser.finalize(1)
+        parser.finalize(0)
+    except KeyboardInterrupt:
+        print('\nCaught keyboard interrupt. Exiting.\n')
+        exit(0)
 
 
 if __name__ == '__main__':
