@@ -236,7 +236,7 @@ class MySQLTestCase(TestCase):
 
     @skipIf(True, 'MySQL module claims this function is not ready for production')
     def test_free_slave(self):
-        self.assertTrue(False)
+        pass
 
     def test_query(self):
         self._test_call(mysql.query, 'SELECT * FROM testdb', 'testdb', 'SELECT * FROM testdb')
