@@ -1,7 +1,6 @@
 # -*- coding: utf-8 -*-
 
 # Import python libs
-import warnings
 
 # Import Salt Testing libs
 from salttesting import skipIf, TestCase
@@ -28,7 +27,9 @@ class PipTestCase(TestCase):
                 expected_cmd,
                 saltenv='base',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
 
     def test_install_editable_withough_egg_fails(self):
@@ -39,7 +40,7 @@ class PipTestCase(TestCase):
                 pip.install,
                 editable='git+https://github.com/saltstack/salt-testing.git'
             )
-            #mock.assert_called_once_with('', runas=None, cwd=None)
+            #mock.assert_called_once_with('', cwd=None, use_vt=False)
 
     def test_install_multiple_editable(self):
         editables = [
@@ -57,7 +58,9 @@ class PipTestCase(TestCase):
                 '--editable=git+https://github.com/saltstack/salt-testing.git#egg=SaltTesting',
                 saltenv='base',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
 
         # Passing editables as a comma separated list
@@ -70,7 +73,9 @@ class PipTestCase(TestCase):
                 '--editable=git+https://github.com/saltstack/salt-testing.git#egg=SaltTesting',
                 saltenv='base',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
 
     def test_install_multiple_pkgs_and_editables(self):
@@ -94,7 +99,9 @@ class PipTestCase(TestCase):
                 '--editable=git+https://github.com/saltstack/salt-testing.git#egg=SaltTesting',
                 saltenv='base',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
 
         # Passing editables as a comma separated list
@@ -107,7 +114,9 @@ class PipTestCase(TestCase):
                 '--editable=git+https://github.com/saltstack/salt-testing.git#egg=SaltTesting',
                 saltenv='base',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
 
         # As a single string
@@ -119,7 +128,9 @@ class PipTestCase(TestCase):
                 '--editable=git+https://github.com/jek/blinker.git#egg=Blinker',
                 saltenv='base',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
 
     def test_issue5940_install_multiple_pip_mirrors(self):
@@ -140,7 +151,9 @@ class PipTestCase(TestCase):
                 '--mirrors=http://pypi.crate.io',
                 saltenv='base',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
 
         # Passing mirrors as a comma separated list
@@ -154,7 +167,9 @@ class PipTestCase(TestCase):
                 '--mirrors=http://pypi.crate.io',
                 saltenv='base',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
 
         # As a single string
@@ -166,7 +181,9 @@ class PipTestCase(TestCase):
                 '--mirrors=http://g.pypi.python.org',
                 saltenv='base',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
 
     def test_install_with_multiple_find_links(self):
@@ -187,7 +204,9 @@ class PipTestCase(TestCase):
                 '--find-links=http://pypi.crate.io \'pep8\'',
                 saltenv='base',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
 
         # Passing mirrors as a comma separated list
@@ -201,7 +220,9 @@ class PipTestCase(TestCase):
                 '--find-links=http://pypi.crate.io \'pep8\'',
                 saltenv='base',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
 
         # Passing mirrors as a single string entry
@@ -212,7 +233,9 @@ class PipTestCase(TestCase):
                 'pip install --find-links=http://g.pypi.python.org \'pep8\'',
                 saltenv='base',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
 
         # Invalid proto raises exception
@@ -242,7 +265,9 @@ class PipTestCase(TestCase):
                 '--find-links=https://pypi.crate.io \'pep8\'',
                 saltenv='base',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
 
     def test_install_no_index_with_index_url_or_extra_index_url_raises(self):
@@ -252,7 +277,7 @@ class PipTestCase(TestCase):
                 CommandExecutionError,
                 pip.install, no_index=True, index_url='http://foo.tld'
             )
-            #mock.assert_called_once_with('', runas=None, cwd=None)
+            #mock.assert_called_once_with('', cwd=None)
 
         mock = MagicMock(return_value={'retcode': 0, 'stdout': ''})
         with patch.dict(pip.__salt__, {'cmd.run_all': mock}):
@@ -260,7 +285,7 @@ class PipTestCase(TestCase):
                 CommandExecutionError,
                 pip.install, no_index=True, extra_index_url='http://foo.tld'
             )
-            #mock.assert_called_once_with('', runas=None, cwd=None)
+            #mock.assert_called_once_with('', cwd=None)
 
     @patch('salt.modules.pip._get_cached_requirements')
     def test_install_failed_cached_requirements(self, get_cached_requirements):
@@ -280,11 +305,13 @@ class PipTestCase(TestCase):
                 expected_cmd,
                 saltenv='base',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
 
     @patch('os.path')
-    def test_install_fix_activate_env(self, mock_path):
+    def test_install_venv(self, mock_path):
         mock_path.is_file.return_value = True
         mock_path.isdir.return_value = True
 
@@ -293,14 +320,17 @@ class PipTestCase(TestCase):
         mock_path.join = join
         mock = MagicMock(return_value={'retcode': 0, 'stdout': ''})
         with patch.dict(pip.__salt__, {'cmd.run_all': mock}):
-            pip.install('mock', bin_env='/test_env', activate=True)
+            pip.install('mock', bin_env='/test_env')
             mock.assert_called_once_with(
-                '. /test_env/bin/activate && /test_env/bin/pip install '
+                '/test_env/bin/pip install '
                 '\'mock\'',
                 env={'VIRTUAL_ENV': '/test_env'},
                 saltenv='base',
                 runas=None,
-                cwd=None)
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
+            )
 
     @patch('os.path')
     def test_install_log_argument_in_resulting_command(self, mock_path):
@@ -311,7 +341,9 @@ class PipTestCase(TestCase):
                 'pip install --log=/tmp/pip-install.log \'pep8\'',
                 saltenv='base',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
 
         # Let's fake a non-writable log file
@@ -334,7 +366,9 @@ class PipTestCase(TestCase):
                 'pip install --timeout=10 \'pep8\'',
                 saltenv='base',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
 
         # Passing an int as a string
@@ -345,7 +379,9 @@ class PipTestCase(TestCase):
                 'pip install --timeout=10 \'pep8\'',
                 saltenv='base',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
 
         # Passing a non-int to timeout
@@ -366,7 +402,9 @@ class PipTestCase(TestCase):
                 'pip install --index-url=\'http://foo.tld\' \'pep8\'',
                 saltenv='base',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
 
     def test_install_extra_index_url_argument_in_resulting_command(self):
@@ -377,7 +415,9 @@ class PipTestCase(TestCase):
                 'pip install --extra-index-url=\'http://foo.tld\' \'pep8\'',
                 saltenv='base',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
 
     def test_install_no_index_argument_in_resulting_command(self):
@@ -388,7 +428,9 @@ class PipTestCase(TestCase):
                 'pip install --no-index \'pep8\'',
                 saltenv='base',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
 
     def test_install_build_argument_in_resulting_command(self):
@@ -399,7 +441,9 @@ class PipTestCase(TestCase):
                 'pip install --build=/tmp/foo \'pep8\'',
                 saltenv='base',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
 
     def test_install_target_argument_in_resulting_command(self):
@@ -410,7 +454,9 @@ class PipTestCase(TestCase):
                 'pip install --target=/tmp/foo \'pep8\'',
                 saltenv='base',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
 
     def test_install_download_argument_in_resulting_command(self):
@@ -421,7 +467,9 @@ class PipTestCase(TestCase):
                 'pip install --download=/tmp/foo \'pep8\'',
                 saltenv='base',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
 
     def test_install_no_download_argument_in_resulting_command(self):
@@ -432,7 +480,9 @@ class PipTestCase(TestCase):
                 'pip install --no-download \'pep8\'',
                 saltenv='base',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
 
     def test_install_download_cache_argument_in_resulting_command(self):
@@ -443,7 +493,9 @@ class PipTestCase(TestCase):
                 'pip install --download-cache=/tmp/foo \'pep8\'',
                 saltenv='base',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
 
     def test_install_source_argument_in_resulting_command(self):
@@ -454,7 +506,9 @@ class PipTestCase(TestCase):
                 'pip install --source=/tmp/foo \'pep8\'',
                 saltenv='base',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
 
     def test_install_exists_action_argument_in_resulting_command(self):
@@ -466,7 +520,9 @@ class PipTestCase(TestCase):
                     'pip install --exists-action={0} \'pep8\''.format(action),
                     saltenv='base',
                     runas=None,
-                    cwd=None
+                    cwd=None,
+                    use_vt=False,
+                    python_shell=False,
                 )
 
         # Test for invalid action
@@ -495,7 +551,9 @@ class PipTestCase(TestCase):
                 '--install-option=\'--install-scripts=/foo/bar/bin\' \'pep8\'',
                 saltenv='base',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
 
         # Passing mirrors as a comma separated list
@@ -508,7 +566,9 @@ class PipTestCase(TestCase):
                 '--install-option=\'--install-scripts=/foo/bar/bin\' \'pep8\'',
                 saltenv='base',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
 
         # Passing mirrors as a single string entry
@@ -520,7 +580,9 @@ class PipTestCase(TestCase):
                 '\'pep8\'',
                 saltenv='base',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
 
     def test_install_global_options_argument_in_resulting_command(self):
@@ -539,7 +601,9 @@ class PipTestCase(TestCase):
                 '--global-option=\'--no-user-cfg\' \'pep8\'',
                 saltenv='base',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
 
         # Passing mirrors as a comma separated list
@@ -552,7 +616,9 @@ class PipTestCase(TestCase):
                 '--global-option=\'--no-user-cfg\' \'pep8\'',
                 saltenv='base',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
 
         # Passing mirrors as a single string entry
@@ -563,7 +629,9 @@ class PipTestCase(TestCase):
                 'pip install --global-option=\'--quiet\' \'pep8\'',
                 saltenv='base',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
 
     def test_install_upgrade_argument_in_resulting_command(self):
@@ -574,7 +642,9 @@ class PipTestCase(TestCase):
                 'pip install --upgrade \'pep8\'',
                 saltenv='base',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
 
     def test_install_force_reinstall_argument_in_resulting_command(self):
@@ -585,7 +655,9 @@ class PipTestCase(TestCase):
                 'pip install --force-reinstall \'pep8\'',
                 saltenv='base',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
 
     def test_install_ignore_installed_argument_in_resulting_command(self):
@@ -596,7 +668,9 @@ class PipTestCase(TestCase):
                 'pip install --ignore-installed \'pep8\'',
                 saltenv='base',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
 
     def test_install_no_deps_argument_in_resulting_command(self):
@@ -607,7 +681,9 @@ class PipTestCase(TestCase):
                 'pip install --no-deps \'pep8\'',
                 saltenv='base',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
 
     def test_install_no_install_argument_in_resulting_command(self):
@@ -618,7 +694,9 @@ class PipTestCase(TestCase):
                 'pip install --no-install \'pep8\'',
                 saltenv='base',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
 
     def test_install_proxy_argument_in_resulting_command(self):
@@ -630,7 +708,9 @@ class PipTestCase(TestCase):
                 '--proxy=\'salt-user:salt-passwd@salt-proxy:3128\' \'pep8\'',
                 saltenv='base',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
 
     @patch('salt.modules.pip._get_cached_requirements')
@@ -652,7 +732,9 @@ class PipTestCase(TestCase):
                 '--requirement=\'my_cached_reqs-2\'',
                 saltenv='base',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
 
         # Passing option as a comma separated list
@@ -668,7 +750,9 @@ class PipTestCase(TestCase):
                 '--requirement=\'my_cached_reqs-2\'',
                 saltenv='base',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
 
         # Passing option as a single string entry
@@ -680,7 +764,9 @@ class PipTestCase(TestCase):
                 'pip install --requirement=\'my_cached_reqs-1\'',
                 saltenv='base',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
 
     @patch('salt.modules.pip._get_cached_requirements')
@@ -702,7 +788,9 @@ class PipTestCase(TestCase):
                 '--requirement=\'my_cached_reqs-2\'',
                 saltenv='base',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
 
         # Passing option as a comma separated list
@@ -718,7 +806,9 @@ class PipTestCase(TestCase):
                 '--requirement=\'my_cached_reqs-2\'',
                 saltenv='base',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
 
         # Passing option as a single string entry
@@ -730,7 +820,9 @@ class PipTestCase(TestCase):
                 'pip uninstall -y --requirement=\'my_cached_reqs-1\'',
                 saltenv='base',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
 
     def test_uninstall_proxy_argument_in_resulting_command(self):
@@ -744,7 +836,9 @@ class PipTestCase(TestCase):
                 '--proxy=\'salt-user:salt-passwd@salt-proxy:3128\' pep8',
                 saltenv='base',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
 
     @patch('os.path')
@@ -756,7 +850,9 @@ class PipTestCase(TestCase):
                 'pip uninstall -y --log=/tmp/pip-install.log pep8',
                 saltenv='base',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
 
         # Let's fake a non-writable log file
@@ -779,7 +875,9 @@ class PipTestCase(TestCase):
                 'pip uninstall -y --timeout=10 pep8',
                 saltenv='base',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
 
         # Passing an int as a string
@@ -790,7 +888,9 @@ class PipTestCase(TestCase):
                 'pip uninstall -y --timeout=10 pep8',
                 saltenv='base',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
 
         # Passing a non-int to timeout
@@ -822,7 +922,9 @@ class PipTestCase(TestCase):
             mock.assert_called_once_with(
                 'pip freeze',
                 runas=None,
-                cwd=None
+                cwd=None,
+                use_vt=False,
+                python_shell=False,
             )
             self.assertEqual(ret, eggs)
 
@@ -842,37 +944,38 @@ class PipTestCase(TestCase):
             'bbfreeze-loader==1.1.0',
             'pycrypto==2.6'
         ]
-        mock = MagicMock(
-            side_effect=[
-                {'retcode': 0, 'stdout': 'pip MOCKED_VERSION'},
-                {'retcode': 0, 'stdout': '\n'.join(eggs)}
-            ]
-        )
+        mock_version = '6.1.1'
+        mock = MagicMock(return_value={'retcode': 0, 'stdout': '\n'.join(eggs)})
         with patch.dict(pip.__salt__, {'cmd.run_all': mock}):
-            ret = pip.list_()
-            mock.assert_called_with(
-                'pip freeze',
-                runas=None,
-                cwd=None
-            )
-            self.assertEqual(
-                ret, {
-                    'SaltTesting-dev': 'git+git@github.com:s0undt3ch/salt-testing.git@9ed81aa2f918d59d3706e56b18f0782d1ea43bf8',
-                    'M2Crypto': '0.21.1',
-                    'bbfreeze-loader': '1.1.0',
-                    'bbfreeze': '1.1.0',
-                    'pip': 'MOCKED_VERSION',
-                    'pycrypto': '2.6'
-                }
-            )
+            with patch('salt.modules.pip.version',
+                       MagicMock(return_value=mock_version)):
+                ret = pip.list_()
+                mock.assert_called_with(
+                    'pip freeze',
+                    runas=None,
+                    cwd=None,
+                    python_shell=False,
+                )
+                self.assertEqual(
+                    ret, {
+                        'SaltTesting-dev': 'git+git@github.com:s0undt3ch/salt-testing.git@9ed81aa2f918d59d3706e56b18f0782d1ea43bf8',
+                        'M2Crypto': '0.21.1',
+                        'bbfreeze-loader': '1.1.0',
+                        'bbfreeze': '1.1.0',
+                        'pip': mock_version,
+                        'pycrypto': '2.6'
+                    }
+                )
 
         # Non zero returncode raises exception?
         mock = MagicMock(return_value={'retcode': 1, 'stderr': 'CABOOOOMMM!'})
         with patch.dict(pip.__salt__, {'cmd.run_all': mock}):
-            self.assertRaises(
-                CommandExecutionError,
-                pip.list_,
-            )
+            with patch('salt.modules.pip.version',
+                       MagicMock(return_value='6.1.1')):
+                self.assertRaises(
+                    CommandExecutionError,
+                    pip.list_,
+                )
 
     def test_list_command_with_prefix(self):
         eggs = [
@@ -893,7 +996,8 @@ class PipTestCase(TestCase):
             mock.assert_called_with(
                 'pip freeze',
                 runas=None,
-                cwd=None
+                cwd=None,
+                python_shell=False,
             )
             self.assertEqual(
                 ret, {
@@ -910,135 +1014,36 @@ class PipTestCase(TestCase):
             {'retcode': 0, 'stdout': ''}
         ])
         with patch.dict(pip.__salt__, {'cmd.run_all': mock}):
-            pip.install(
-                'pep8', pre_releases=True
-            )
-            mock.assert_called_with(
-                'pip install \'pep8\'',
-                saltenv='base',
-                runas=None,
-                cwd=None
-            )
-
-        mock = MagicMock(side_effect=[
-            {'retcode': 0, 'stdout': 'pip 1.4.0 /path/to/site-pacakges/pip'},
-            {'retcode': 0, 'stdout': ''}
-        ])
-        with patch.dict(pip.__salt__, {'cmd.run_all': mock}):
-            pip.install(
-                'pep8', pre_releases=True
-            )
-            mock.assert_called_with(
-                'pip install --pre \'pep8\'',
-                saltenv='base',
-                runas=None,
-                cwd=None
-            )
-
-    def test_install_deprecated_runas_triggers_warning(self):
-        # We *always* want *all* warnings thrown on this module
-        warnings.resetwarnings()
-        warnings.filterwarnings('always', '', DeprecationWarning, __name__)
-
-        mock = MagicMock(return_value={'retcode': 0, 'stdout': ''})
-        with patch.dict(pip.__salt__, {'cmd.run_all': mock}):
-            with warnings.catch_warnings(record=True) as w:
-                pip.install('pep8', runas='me!')
-                self.assertEqual(
-                    'The \'runas\' argument to pip.install is deprecated, and '
-                    'will be removed in Salt Lithium (Unreleased). Please '
-                    'use \'user\' instead.', str(w[-1].message)
+            with patch('salt.modules.pip.version',
+                       MagicMock(return_value='1.3')):
+                pip.install(
+                    'pep8', pre_releases=True
+                )
+                mock.assert_called_with(
+                    'pip install \'pep8\'',
+                    saltenv='base',
+                    runas=None,
+                    cwd=None,
+                    use_vt=False,
+                    python_shell=False,
                 )
 
-    def test_uninstall_deprecated_runas_triggers_warning(self):
-        # We *always* want *all* warnings thrown on this module
-        warnings.resetwarnings()
-        warnings.filterwarnings('always', '', DeprecationWarning, __name__)
-
-        mock = MagicMock(return_value={'retcode': 0, 'stdout': ''})
-        with patch.dict(pip.__salt__, {'cmd.run_all': mock}):
-            with warnings.catch_warnings(record=True) as w:
-                pip.uninstall('pep8', runas='me!')
-                self.assertEqual(
-                    'The \'runas\' argument to pip.install is deprecated, and '
-                    'will be removed in Salt Lithium (Unreleased). Please '
-                    'use \'user\' instead.', str(w[-1].message)
+        mock_run = MagicMock(return_value='pip 1.4.1 /path/to/site-packages/pip')
+        mock_run_all = MagicMock(return_value={'retcode': 0, 'stdout': ''})
+        with patch.dict(pip.__salt__, {'cmd.run': mock_run,
+                                       'cmd.run_all': mock_run_all}):
+            with patch('salt.modules.pip._get_pip_bin',
+                       MagicMock(return_value='pip')):
+                pip.install('pep8', pre_releases=True)
+                mock_run_all.assert_called_with(
+                    'pip install --pre \'pep8\'',
+                    saltenv='base',
+                    runas=None,
+                    cwd=None,
+                    use_vt=False,
+                    python_shell=False,
                 )
 
-    def test_freeze_deprecated_runas_triggers_warning(self):
-        # We *always* want *all* warnings thrown on this module
-        warnings.resetwarnings()
-        warnings.filterwarnings('always', '', DeprecationWarning, __name__)
-
-        mock = MagicMock(return_value={'retcode': 0, 'stdout': ''})
-        with patch.dict(pip.__salt__, {'cmd.run_all': mock}):
-            with warnings.catch_warnings(record=True) as w:
-                pip.freeze('/tmp/pip-env', runas='me!')
-                self.assertEqual(
-                    'The \'runas\' argument to pip.install is deprecated, and '
-                    'will be removed in Salt Lithium (Unreleased). Please '
-                    'use \'user\' instead.', str(w[-1].message)
-                )
-
-    def test_list_deprecated_runas_triggers_warning(self):
-        # We *always* want *all* warnings thrown on this module
-        warnings.resetwarnings()
-        warnings.filterwarnings('always', '', DeprecationWarning, __name__)
-
-        mock = MagicMock(return_value={'retcode': 0, 'stdout': 'pip VERSION'})
-        with patch.dict(pip.__salt__, {'cmd.run_all': mock}):
-            with warnings.catch_warnings(record=True) as w:
-                pip.list_('blah', runas='me!')
-                self.assertEqual(
-                    'The \'runas\' argument to pip.install is deprecated, and '
-                    'will be removed in Salt Lithium (Unreleased). Please '
-                    'use \'user\' instead.', str(w[-1].message)
-                )
-
-    def test_install_user_and_runas_raises_exception(self):
-        mock = MagicMock(return_value={'retcode': 0, 'stdout': ''})
-        with patch.dict(pip.__salt__, {'cmd.run_all': mock}):
-            self.assertRaises(
-                CommandExecutionError,
-                pip.install,
-                'pep8',
-                user='Me!',
-                runas='Not Me!'
-            )
-
-    def test_uninstall_user_and_runas_raises_exception(self):
-        mock = MagicMock(return_value={'retcode': 0, 'stdout': ''})
-        with patch.dict(pip.__salt__, {'cmd.run_all': mock}):
-            self.assertRaises(
-                CommandExecutionError,
-                pip.uninstall,
-                'pep8',
-                user='Me!',
-                runas='Not Me!'
-            )
-
-    def test_freeze_user_and_runas_raises_exception(self):
-        mock = MagicMock(return_value={'retcode': 0, 'stdout': ''})
-        with patch.dict(pip.__salt__, {'cmd.run_all': mock}):
-            self.assertRaises(
-                CommandExecutionError,
-                pip.freeze,
-                '/tmp/pip-env',
-                user='Me!',
-                runas='Not Me!'
-            )
-
-    def test_list_user_and_runas_raises_exception(self):
-        mock = MagicMock(return_value={'retcode': 0, 'stdout': ''})
-        with patch.dict(pip.__salt__, {'cmd.run_all': mock}):
-            self.assertRaises(
-                CommandExecutionError,
-                pip.list_,
-                'pep8',
-                user='Me!',
-                runas='Not Me!'
-            )
-
 
 if __name__ == '__main__':
     from integration import run_tests
