@@ -1,17 +1,12 @@
 # -*- coding: utf-8 -*-
 '''
     :codeauthor: :email:`Pedro Algarvio (pedro@algarvio.me)`
-    :copyright: © 2013 by the SaltStack Team, see AUTHORS for more details
-    :license: Apache 2.0, see LICENSE for more details.
 
 
     tests.unit.states.pip_test
     ~~~~~~~~~~~~~~~~~~~~~~~~~~
 '''
 
-# Import python libs
-import warnings
-
 # Import Salt Testing libs
 from salttesting import skipIf, TestCase
 from salttesting.helpers import ensure_in_syspath
@@ -21,7 +16,6 @@ ensure_in_syspath('../../')
 # Import salt libs
 import integration
 from salt.states import pip_state
-from salt.exceptions import CommandExecutionError
 
 # Import 3rd-party libs
 try:
@@ -40,82 +34,6 @@ pip_state.__salt__ = {'cmd.which_bin': lambda _: 'pip'}
         'The \'pip\' library is not importable(installed system-wide)')
 class PipStateTest(TestCase, integration.SaltReturnAssertsMixIn):
 
-    def test_installed_deprecated_runas(self):
-        # We *always* want *all* warnings thrown on this module
-        warnings.resetwarnings()
-        warnings.filterwarnings('always', '', DeprecationWarning, __name__)
-
-        mock = MagicMock(return_value={'retcode': 0, 'stdout': ''})
-        pip_list = MagicMock(return_value=[])
-        pip_install = MagicMock(return_value={'retcode': 0})
-        with patch.dict(pip_state.__salt__, {'cmd.run_all': mock,
-                                             'pip.list': pip_list,
-                                             'pip.install': pip_install}):
-            with warnings.catch_warnings(record=True) as w:
-                ret = pip_state.installed('pep8', runas='me!')
-                self.assertEqual(
-                    'The \'runas\' argument to pip.installed is deprecated, '
-                    'and will be removed in Salt Lithium (Unreleased). '
-                    'Please use \'user\' instead.', str(w[-1].message)
-                )
-                self.assertSaltTrueReturn({'testsuite': ret})
-                # Is the state returning a warnings key with the deprecation
-                # message?
-                self.assertInSalStatetWarning(
-                    'The \'runas\' argument to pip.installed is deprecated, '
-                    'and will be removed in Salt Lithium (Unreleased). '
-                    'Please use \'user\' instead.', {'testsuite': ret}
-                )
-
-    def test_installed_runas_and_user_raises_exception(self):
-        mock = MagicMock(return_value={'retcode': 0, 'stdout': ''})
-        with patch.dict(pip_state.__salt__, {'cmd.run_all': mock}):
-            self.assertRaises(
-                CommandExecutionError,
-                pip_state.installed,
-                'pep8',
-                user='Me!',
-                runas='Not Me!'
-            )
-
-    def test_removed_deprecated_runas(self):
-        # We *always* want *all* warnings thrown on this module
-        warnings.resetwarnings()
-        warnings.filterwarnings('always', '', DeprecationWarning, __name__)
-
-        mock = MagicMock(return_value={'retcode': 0, 'stdout': ''})
-        pip_list = MagicMock(return_value=['pep8'])
-        pip_uninstall = MagicMock(return_value=True)
-        with patch.dict(pip_state.__salt__, {'cmd.run_all': mock,
-                                             'pip.list': pip_list,
-                                             'pip.uninstall': pip_uninstall}):
-            with warnings.catch_warnings(record=True) as w:
-                ret = pip_state.removed('pep8', runas='me!')
-                self.assertEqual(
-                    'The \'runas\' argument to pip.installed is deprecated, '
-                    'and will be removed in Salt Lithium (Unreleased). '
-                    'Please use \'user\' instead.', str(w[-1].message)
-                )
-                self.assertSaltTrueReturn({'testsuite': ret})
-                # Is the state returning a warnings key with the deprecation
-                # message?
-                self.assertInSalStatetWarning(
-                    'The \'runas\' argument to pip.installed is deprecated, '
-                    'and will be removed in Salt Lithium (Unreleased). '
-                    'Please use \'user\' instead.', {'testsuite': ret}
-                )
-
-    def test_removed_runas_and_user_raises_exception(self):
-        mock = MagicMock(return_value={'retcode': 0, 'stdout': ''})
-        with patch.dict(pip_state.__salt__, {'cmd.run_all': mock}):
-            self.assertRaises(
-                CommandExecutionError,
-                pip_state.removed,
-                'pep8',
-                user='Me!',
-                runas='Not Me!'
-            )
-
     def test_install_requirements_parsing(self):
         mock = MagicMock(return_value={'retcode': 0, 'stdout': ''})
         pip_list = MagicMock(return_value={'pep8': '1.3.3'})
@@ -132,13 +50,15 @@ class PipStateTest(TestCase, integration.SaltReturnAssertsMixIn):
 
         mock = MagicMock(return_value={'retcode': 0, 'stdout': ''})
         pip_list = MagicMock(return_value={'pep8': '1.3.3'})
+        pip_install = MagicMock(return_value={'retcode': 0})
         with patch.dict(pip_state.__salt__, {'cmd.run_all': mock,
-                                             'pip.list': pip_list}):
+                                             'pip.list': pip_list,
+                                             'pip.install': pip_install}):
             with patch.dict(pip_state.__opts__, {'test': True}):
                 ret = pip_state.installed('pep8>=1.3.2')
                 self.assertSaltTrueReturn({'test': ret})
                 self.assertInSaltComment(
-                    'Python package pep8>=1.3.2 already installed',
+                    'Python package pep8>=1.3.2 was already installed',
                     {'test': ret}
                 )
 
@@ -156,25 +76,29 @@ class PipStateTest(TestCase, integration.SaltReturnAssertsMixIn):
 
         mock = MagicMock(return_value={'retcode': 0, 'stdout': ''})
         pip_list = MagicMock(return_value={'pep8': '1.3.2'})
+        pip_install = MagicMock(return_value={'retcode': 0})
         with patch.dict(pip_state.__salt__, {'cmd.run_all': mock,
-                                             'pip.list': pip_list}):
+                                             'pip.list': pip_list,
+                                             'pip.install': pip_install}):
             with patch.dict(pip_state.__opts__, {'test': True}):
                 ret = pip_state.installed('pep8>1.3.1,<1.3.3')
                 self.assertSaltTrueReturn({'test': ret})
                 self.assertInSaltComment(
-                    'Python package pep8>1.3.1;<1.3.3 already installed',
+                    'Python package pep8>1.3.1,<1.3.3 was already installed',
                     {'test': ret}
                 )
 
         mock = MagicMock(return_value={'retcode': 0, 'stdout': ''})
         pip_list = MagicMock(return_value={'pep8': '1.3.1'})
+        pip_install = MagicMock(return_value={'retcode': 0})
         with patch.dict(pip_state.__salt__, {'cmd.run_all': mock,
-                                             'pip.list': pip_list}):
+                                             'pip.list': pip_list,
+                                             'pip.install': pip_install}):
             with patch.dict(pip_state.__opts__, {'test': True}):
                 ret = pip_state.installed('pep8>1.3.1,<1.3.3')
                 self.assertSaltNoneReturn({'test': ret})
                 self.assertInSaltComment(
-                    'Python package pep8>1.3.1;<1.3.3 is set to be installed',
+                    'Python package pep8>1.3.1,<1.3.3 is set to be installed',
                     {'test': ret}
                 )
 
@@ -250,11 +174,7 @@ class PipStateTest(TestCase, integration.SaltReturnAssertsMixIn):
                 '#md5=e6760af92b7165f8be53b5763e40bc24'
             )
             self.assertSaltTrueReturn({'test': ret})
-            self.assertInSaltComment(
-                'There was no error installing package '
-                '\'https://pypi.python.org/packages/source/S/SaltTesting/'
-                'SaltTesting-0.5.0.tar.gz#md5=e6760af92b7165f8be53b5763e40bc24\' '
-                'although it does not show when calling \'pip.freeze\'.',
+            self.assertInSaltComment('All packages were successfully installed',
                 {'test': ret}
             )
             self.assertInSaltReturn(
@@ -266,7 +186,7 @@ class PipStateTest(TestCase, integration.SaltReturnAssertsMixIn):
             )
 
         mock = MagicMock(return_value={'retcode': 0, 'stdout': ''})
-        pip_list = MagicMock(return_value={'pep8': '1.3.1'})
+        pip_list = MagicMock(return_value={'SaltTesting': '0.5.0'})
         pip_install = MagicMock(return_value={
             'retcode': 0,
             'stderr': '',
@@ -281,7 +201,7 @@ class PipStateTest(TestCase, integration.SaltReturnAssertsMixIn):
                 )
                 self.assertSaltTrueReturn({'test': ret})
                 self.assertInSaltComment(
-                    'Package was successfully installed',
+                    'successfully installed',
                     {'test': ret}
                 )
 
@@ -300,7 +220,7 @@ class PipStateTest(TestCase, integration.SaltReturnAssertsMixIn):
 
         # Test VCS installations using git+git://
         mock = MagicMock(return_value={'retcode': 0, 'stdout': ''})
-        pip_list = MagicMock(return_value={'pep8': '1.3.1'})
+        pip_list = MagicMock(return_value={'SaltTesting': '0.5.0'})
         pip_install = MagicMock(return_value={
             'retcode': 0,
             'stderr': '',
@@ -315,13 +235,13 @@ class PipStateTest(TestCase, integration.SaltReturnAssertsMixIn):
                 )
                 self.assertSaltTrueReturn({'test': ret})
                 self.assertInSaltComment(
-                    'Package was successfully installed',
+                    'were successfully installed',
                     {'test': ret}
                 )
 
         # Test VCS installations with version info like >= 0.1
         try:
-            orignal_pip_version = pip.__version__
+            original_pip_version = pip.__version__
             pip.__version__ = MagicMock(
                 side_effect=AttributeError(
                     'Faked missing __version__ attribute'
@@ -332,7 +252,7 @@ class PipStateTest(TestCase, integration.SaltReturnAssertsMixIn):
             pass
 
         mock = MagicMock(return_value={'retcode': 0, 'stdout': ''})
-        pip_list = MagicMock(return_value={'pep8': '1.3.1'})
+        pip_list = MagicMock(return_value={'SaltTesting': '0.5.0'})
         pip_install = MagicMock(return_value={
             'retcode': 0,
             'stderr': '',
@@ -347,13 +267,13 @@ class PipStateTest(TestCase, integration.SaltReturnAssertsMixIn):
                 )
                 self.assertSaltTrueReturn({'test': ret})
                 self.assertInSaltComment(
-                    'Package was successfully installed',
+                    'were successfully installed',
                     {'test': ret}
                 )
 
         # Reset the version attribute if existing
         if hasattr(pip, '__version__'):
-            pip.__version__ = orignal_pip_version
+            pip.__version__ = original_pip_version
 
 
 if __name__ == '__main__':
